'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _noop = require('lodash/noop');

var _noop2 = _interopRequireDefault(_noop);

var _isNull = require('lodash/isNull');

var _isNull2 = _interopRequireDefault(_isNull);

var _Popup = require('../Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _DatePickerPopup = require('./DatePickerPopup');

var _DatePickerPopup2 = _interopRequireDefault(_DatePickerPopup);

var _WithIdAndTypeHOC = require('../util/hoc/WithIdAndTypeHOC');

var _dateUtils = require('../util/date-utils');

var _dateFormats = require('../util/date-formats');

var _Input = require('../Input');

var _Input2 = _interopRequireDefault(_Input);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    disabled: _propTypes2.default.bool,
    className: _propTypes2.default.string,
    value: _propTypes2.default.string,
    format: _propTypes2.default.string,
    onChange: _propTypes2.default.func,
    minDate: _propTypes2.default.string,
    maxDate: _propTypes2.default.string,
    attachment: _propTypes2.default.oneOf(Object.keys(_Popup2.default.attachmentPositions))
};

var DEFAULT_PROPS = {
    onChange: _noop2.default,
    format: _dateUtils.DD_MM_YYYY,
    attachment: 'bottom center',
    disabled: false
};

var DatePicker = function (_React$Component) {
    _inherits(DatePicker, _React$Component);

    function DatePicker(props) {
        _classCallCheck(this, DatePicker);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.formatValue = function () {
            if (!_this.props.value) {
                return '';
            }

            var momentValue = _this.getMomentValue(_this.props.value);

            if ((0, _isNull2.default)(momentValue)) {
                return _this.props.value;
            }

            return (0, _dateFormats.formatDMonthYear)(momentValue);
        };

        _this._handleInputChange = function (value) {
            var moment = _this.getMomentValue(value);
            if (!(0, _isNull2.default)(moment)) {
                _this.resetMonth(value);
            }

            _this.props.onChange(value);
        };

        _this.resetMonth = function (value) {
            var momentValue = _this.getMomentValue(value);
            var month = (0, _isNull2.default)(momentValue) ? (0, _dateUtils.getToday)() : momentValue;

            _this.handleMonthChange((0, _dateUtils.getFirstDayOfMonth)(month));
        };

        _this.getMomentValue = function (value) {
            var parsed = _this.doParse(value);

            if (parsed.isValid()) {
                return parsed;
            }

            return null;
        };

        _this.handleMonthChange = function (month) {
            _this.setState({ month: month });
        };

        _this.handlePopupStateChange = function (open) {
            if (_this.props.disabled) {
                return;
            }

            _this.setState({ open: open });
        };

        _this.state = {
            open: false
        };
        _this._triggerOnChange = (0, _debounce2.default)(_this._triggerOnChange, 50).bind(_this);
        _this._handleSelection = _this._handleSelection.bind(_this);
        return _this;
    }

    DatePicker.prototype._handleSelection = function _handleSelection(selected) {
        var _this2 = this;

        this.setState({
            open: false
        }, function () {
            setTimeout(function () {
                _this2.props.onChange(_this2.doFormat(selected));
            }, 200);
        });
    };

    DatePicker.prototype.doFormat = function doFormat(date) {
        return (0, _dateUtils.format)(date, this.props.format);
    };

    DatePicker.prototype.doParse = function doParse(date) {
        return (0, _dateUtils.parse)(date, this.props.format);
    };

    DatePicker.prototype.getStyles = function getStyles(date) {
        return (0, _classnames2.default)('date-picker__date-cell', {
            'date-picker__date-cell--selected': (0, _dateUtils.isDayTheSame)(date, this.getMomentValue(this.props.value)),
            'date-picker__date-cell--of-other-month': !(0, _dateUtils.isMonthTheSame)(date, this.state.month)
        });
    };

    DatePicker.prototype._triggerOnChange = function _triggerOnChange(value) {
        this.props.onChange(value);
    };

    DatePicker.prototype._getTarget = function _getTarget() {
        var target = _react2.default.Children.only(this.props.children);
        this.isInput = target.type === _Input2.default;

        return _react2.default.cloneElement(target, {
            id: this.props.id,
            value: this.isInput ? this.props.value || '' : this.formatValue(),
            onChange: this._handleInputChange,
            disabled: this.props.disabled
        });
    };

    DatePicker.prototype.render = function render() {
        var _this3 = this;

        return _react2.default.createElement(
            _Popup2.default,
            {
                popupClassName: 'popover-colored',
                attachment: this.props.attachment,
                on: 'focus',
                onPopupStateChange: this.handlePopupStateChange,
                open: this.state.open,
                onTransitionClosedToOpen: function onTransitionClosedToOpen() {
                    _this3.resetMonth(_this3.props.value);
                }
            },
            this._getTarget(),
            _react2.default.createElement(_DatePickerPopup2.default, {
                month: this.state.month,
                format: this.props.format,
                onSelected: this._handleSelection,
                onMonthChange: this.handleMonthChange,
                minDate: this.props.minDate ? this.doParse(this.props.minDate) : undefined,
                maxDate: this.props.maxDate ? this.doParse(this.props.maxDate) : undefined,
                getStyles: function getStyles(date) {
                    return _this3.getStyles(date);
                }
            })
        );
    };

    return DatePicker;
}(_react2.default.Component);

DatePicker.propTypes = PROPERTY_TYPES;
DatePicker.defaultProps = DEFAULT_PROPS;

exports.default = (0, _WithIdAndTypeHOC.withIdAndTypeContext)(DatePicker, 'DatePicker');