'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _values = require('lodash/values');

var _values2 = _interopRequireDefault(_values);

var _memoize = require('lodash/memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _chunk = require('lodash/chunk');

var _chunk2 = _interopRequireDefault(_chunk);

var _dateUtils = require('../../util/date-utils');

var _dateFormats = require('../../util/date-formats');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var START_DATE_TYPES = {
    NORMALIZED: 'normalized',
    SIMPLE: 'simple'
};

var WEEKS_IN_CALENDAR = 6;
var DAYS_IN_WEEK = 7;

var PROPERTY_TYPES = {
    startDateType: _propTypes2.default.oneOf((0, _values2.default)(START_DATE_TYPES)),
    renderDatesOfOtherMonth: _propTypes2.default.bool,
    getStyles: _propTypes2.default.func,
    onSelected: _propTypes2.default.func,
    onMouseEnter: _propTypes2.default.func,
    onMouseLeave: _propTypes2.default.func,
    formatDate: _propTypes2.default.func,
    formatDataDate: _propTypes2.default.func,
    parseDataDate: _propTypes2.default.func,
    formatWeekDay: _propTypes2.default.func,
    month: _propTypes2.default.object.isRequired,
    componentWillUnmount: _propTypes2.default.func,
    minDate: _propTypes2.default.object,
    maxDate: _propTypes2.default.object
};

var DEFAULT_PROPS = {
    startDateType: START_DATE_TYPES.NORMALIZED,
    renderDatesOfOtherMonth: true,
    formatDate: _dateFormats.formatD,
    formatDataDate: _dateFormats.formatDDMMYYYY,
    parseDataDate: _dateFormats.parseDDMMYYYY,
    formatWeekDay: _dateFormats.formatWeekDay
};

var DatePickerCalendar = function (_React$Component) {
    _inherits(DatePickerCalendar, _React$Component);

    function DatePickerCalendar(props) {
        _classCallCheck(this, DatePickerCalendar);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.renderDaysOfWeek = (0, _memoize2.default)(_this.renderDaysOfWeek);
        return _this;
    }

    DatePickerCalendar.prototype.getStart = function getStart() {
        var firstDayOfMonth = (0, _dateUtils.getFirstDayOfMonth)(this.props.month);
        if (this.props.startDateType === START_DATE_TYPES.NORMALIZED) {
            if ((0, _dateUtils.isFirstDayOfWeek)(firstDayOfMonth)) {
                return (0, _dateUtils.getFirstDayOfPreviousWeek)(firstDayOfMonth);
            } else {
                return (0, _dateUtils.getFirstDayOfWeek)(firstDayOfMonth);
            }
        }
        return (0, _dateUtils.getFirstDayOfWeek)(firstDayOfMonth);
    };

    DatePickerCalendar.prototype.getDates = function getDates() {
        var rollable = this.getStart();
        var dates = [];
        for (var day = 0; day < WEEKS_IN_CALENDAR * DAYS_IN_WEEK; day++) {
            dates.push(rollable);
            rollable = (0, _dateUtils.addDays)(rollable, 1);
        }
        return dates;
    };

    DatePickerCalendar.prototype.componentWillUnmount = function componentWillUnmount() {
        if (this.props.componentWillUnmount) {
            this.props.componentWillUnmount();
        }
    };

    DatePickerCalendar.prototype.retrieveDate = function retrieveDate(e) {
        var selected = e.currentTarget.getAttribute('data-date');
        return this.parseDataDate(selected);
    };

    DatePickerCalendar.prototype.handleClick = function handleClick(e) {
        var date = this.retrieveDate(e);
        if (this.isDisabled(date)) {
            return;
        }

        if (this.props.onSelected) {
            this.props.onSelected(date);
        }
    };

    DatePickerCalendar.prototype.handleMouseEnter = function handleMouseEnter(e) {
        if (this.props.onMouseEnter) {
            this.props.onMouseEnter(this.retrieveDate(e));
        }
    };

    DatePickerCalendar.prototype.handleMouseLeave = function handleMouseLeave(e) {
        if (this.props.onMouseLeave) {
            this.props.onMouseLeave();
        }
    };

    DatePickerCalendar.prototype.getDateStyles = function getDateStyles(date) {
        if (this.props.getStyles) {
            return this.props.getStyles(date);
        }
    };

    DatePickerCalendar.prototype.parseDataDate = function parseDataDate(date) {
        return this.props.parseDataDate(date);
    };

    DatePickerCalendar.prototype.formatDataDate = function formatDataDate(date) {
        return this.props.formatDataDate(date);
    };

    DatePickerCalendar.prototype.formatDate = function formatDate(date) {
        return this.props.formatDate(date);
    };

    DatePickerCalendar.prototype.renderDaysOfWeek = function renderDaysOfWeek() {
        var result = [];

        for (var i = 1; i <= 7; i++) {
            var weekday = (0, _dateUtils.getWeekDayByIndex)(i);
            var formattedWeekday = this.props.formatWeekDay(weekday);
            var isNonWorkingDay = (0, _dateUtils.isWeekend)(weekday);

            var className = (0, _classnames2.default)('date-picker-calendar__day-of-week-cell', {
                'date-picker-calendar__day-of-week-cell--non-working': isNonWorkingDay,
                'date-picker-calendar__day-of-week-cell--working': !isNonWorkingDay
            });
            result.push(_react2.default.createElement(
                'td',
                { key: formattedWeekday, className: className },
                formattedWeekday
            ));
        }
        return result;
    };

    DatePickerCalendar.prototype.renderDates = function renderDates() {
        var _this2 = this;

        var weeks = (0, _chunk2.default)(this.getDates(), DAYS_IN_WEEK);
        return weeks.map(function (week, index) {
            return _react2.default.createElement(
                'tr',
                { key: 'week-' + index, className: 'date-picker-week' },
                _this2.renderWeek(week)
            );
        });
    };

    DatePickerCalendar.prototype.isDisabled = function isDisabled(date) {
        if (this.props.minDate && (0, _dateUtils.isDayBefore)(date, this.props.minDate)) {
            return true;
        }

        if (this.props.maxDate && (0, _dateUtils.isDayAfter)(date, this.props.maxDate)) {
            return true;
        }
    };

    DatePickerCalendar.prototype.renderWeek = function renderWeek(week) {
        var _this3 = this;

        return week.map(function (rollable) {
            var isSameMonth = (0, _dateUtils.isMonthTheSame)(_this3.props.month, rollable);
            var className = 'date-picker-calendar__date-cell';
            var formattedDate = _this3.formatDataDate(rollable);

            if (!_this3.props.renderDatesOfOtherMonth && !isSameMonth) {
                return _react2.default.createElement(
                    'td',
                    { key: formattedDate, 'data-disabled': 'disabled', className: className },
                    _react2.default.createElement(
                        'div',
                        null,
                        '\xA0'
                    )
                );
            } else {
                var dateStyles = _this3.getDateStyles(rollable);

                className = (0, _classnames2.default)(className, dateStyles, {
                    'date-picker-calendar__date-cell--today': (0, _dateUtils.isToday)(rollable)
                });

                var props = {
                    key: formattedDate,
                    onClick: function onClick(e) {
                        return _this3.handleClick(e);
                    },
                    onMouseEnter: function onMouseEnter(e) {
                        return _this3.handleMouseEnter(e);
                    },
                    onMouseLeave: function onMouseLeave(e) {
                        return _this3.handleMouseLeave(e);
                    },
                    className: className,
                    'data-date': formattedDate
                };

                if (_this3.isDisabled(rollable)) {
                    props['data-disabled'] = true;
                }

                return _react2.default.createElement(
                    'td',
                    props,
                    _react2.default.createElement(
                        'div',
                        null,
                        _this3.formatDate(rollable)
                    )
                );
            }
        });
    };

    DatePickerCalendar.prototype.render = function render() {
        return _react2.default.createElement(
            'div',
            { className: 'date-picker-calendar' },
            _react2.default.createElement(
                'table',
                { className: 'date-picker-calendar__days-of-week' },
                _react2.default.createElement(
                    'tbody',
                    null,
                    _react2.default.createElement(
                        'tr',
                        null,
                        this.renderDaysOfWeek()
                    )
                )
            ),
            _react2.default.createElement(
                'table',
                { className: 'date-picker-calendar__grid' },
                _react2.default.createElement(
                    'tbody',
                    null,
                    this.renderDates()
                )
            )
        );
    };

    return DatePickerCalendar;
}(_react2.default.Component);

DatePickerCalendar.propTypes = PROPERTY_TYPES;
DatePickerCalendar.defaultProps = DEFAULT_PROPS;

exports.default = DatePickerCalendar;