'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _range = require('lodash/range');

var _range2 = _interopRequireDefault(_range);

var _Select = require('../../Select/Select');

var _Select2 = _interopRequireDefault(_Select);

var _DatePickerCalendar = require('../DatePickerCalendar');

var _DatePickerCalendar2 = _interopRequireDefault(_DatePickerCalendar);

var _Button = require('../../Button/Button');

var _Button2 = _interopRequireDefault(_Button);

var _dateUtils = require('../../util/date-utils');

var _dateFormats = require('../../util/date-formats');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    month: _propTypes2.default.object, //moment
    onSelected: _propTypes2.default.func.isRequired,
    onMonthChange: _propTypes2.default.func,
    minDate: _propTypes2.default.object,
    maxDate: _propTypes2.default.object,
    getStyles: _propTypes2.default.func,
    format: _propTypes2.default.string.isRequired
};

var DEFAULT_PROPS = {
    month: (0, _dateUtils.getFirstDayOfMonth)((0, _dateUtils.getToday)()),
    minDate: (0, _dateUtils.getFirstDayOfYear)((0, _dateUtils.addYears)((0, _dateUtils.getToday)(), -3)),
    maxDate: (0, _dateUtils.getLastDayOfYear)((0, _dateUtils.addYears)((0, _dateUtils.getToday)(), 2))
};

var DatePickerPopup = function (_React$Component) {
    _inherits(DatePickerPopup, _React$Component);

    function DatePickerPopup(props) {
        _classCallCheck(this, DatePickerPopup);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.years = (0, _range2.default)((0, _dateUtils.getYear)(props.minDate), (0, _dateUtils.getYear)(props.maxDate) + 1).map(function (item) {
            return { id: item, displayString: '' + item };
        });
        return _this;
    }

    DatePickerPopup.prototype.left = function left() {
        if ((0, _dateUtils.isMonthBeforeOrTheSame)(this.props.month, this.props.minDate)) {
            return;
        }
        var month = (0, _dateUtils.addMonths)(this.props.month, -1);
        this.updateMonth(month);
    };

    DatePickerPopup.prototype.right = function right() {
        if ((0, _dateUtils.isMonthAfterOrTheSame)(this.props.month, this.props.maxDate)) {
            return;
        }
        var month = (0, _dateUtils.addMonths)(this.props.month, 1);
        this.updateMonth(month);
    };

    DatePickerPopup.prototype.onYearChange = function onYearChange(selectedYear) {
        var month = (0, _dateUtils.setYear)(this.props.month, selectedYear);
        if ((0, _dateUtils.isMonthBefore)(month, this.props.minDate)) {
            month = this.props.minDate;
        } else if ((0, _dateUtils.isMonthAfter)(month, this.props.maxDate)) {
            month = this.props.maxDate;
        }
        this.updateMonth(month);
    };

    DatePickerPopup.prototype.onMonthChange = function onMonthChange(selectedMonth) {
        var month = (0, _dateUtils.setMonth)(this.props.month, selectedMonth);
        this.updateMonth(month);
    };

    DatePickerPopup.prototype.updateMonth = function updateMonth(month) {
        this.props.onMonthChange(month);
    };

    DatePickerPopup.prototype.getSelectedMonth = function getSelectedMonth() {
        return (0, _dateUtils.getMonth)(this.props.month);
    };

    DatePickerPopup.prototype.getSelectedYear = function getSelectedYear() {
        return (0, _dateUtils.getYear)(this.props.month);
    };

    DatePickerPopup.prototype.getMonthOptions = function getMonthOptions() {
        var result = [];
        var isMinYear = (0, _dateUtils.isYearTheSame)(this.props.minDate, this.props.month);
        var isMaxYear = (0, _dateUtils.isYearTheSame)(this.props.maxDate, this.props.month);

        if (!isMinYear && !isMaxYear) {
            return (0, _dateUtils.getMonths)().map(function (item, index) {
                return { id: index, displayString: '' + item };
            });
        }

        var start = isMinYear ? this.props.minDate : (0, _dateUtils.getFirstDayOfYear)(this.props.maxDate);
        var end = isMaxYear ? this.props.maxDate : (0, _dateUtils.getLastDayOfYear)(this.props.minDate);

        var rollable = start;
        while ((0, _dateUtils.isMonthBeforeOrTheSame)(rollable, end)) {
            result.push({
                id: (0, _dateUtils.getMonth)(rollable),
                displayString: (0, _dateFormats.formatFullMonth)(rollable)
            });
            rollable = (0, _dateUtils.addMonths)(rollable, 1);
        }

        return result;
    };

    DatePickerPopup.prototype.render = function render() {
        var _this2 = this;

        return _react2.default.createElement(
            'div',
            { tabIndex: '-1' },
            _react2.default.createElement(
                'div',
                { className: 'date-picker-popup__header' },
                _react2.default.createElement('i', {
                    onClick: function onClick() {
                        return _this2.left();
                    },
                    className: 'im date-picker-popup__arrow date-picker-popup__arrow--left'
                }),
                _react2.default.createElement(_Select2.default, {
                    className: 'date-picker-popup__months select--sm',
                    value: this.getSelectedMonth(),
                    onChange: function onChange(month) {
                        _this2.onMonthChange(month);
                    },
                    options: this.getMonthOptions()
                }),
                _react2.default.createElement(_Select2.default, {
                    className: 'date-picker-popup__years select--sm',
                    value: this.getSelectedYear(),
                    onChange: function onChange(year) {
                        _this2.onYearChange(year);
                    },
                    options: this.years
                }),
                _react2.default.createElement('i', {
                    onClick: function onClick() {
                        return _this2.right();
                    },
                    className: 'im date-picker-popup__arrow date-picker-popup__arrow--right'
                })
            ),
            _react2.default.createElement(_DatePickerCalendar2.default, {
                onSelected: this.props.onSelected,
                month: this.props.month,
                minDate: this.props.minDate,
                maxDate: this.props.maxDate,
                getStyles: this.props.getStyles
            }),
            _react2.default.createElement(
                _Button2.default,
                {
                    onClick: function onClick() {
                        return _this2.props.onSelected((0, _dateUtils.getToday)());
                    },
                    className: 'date-picker__today-link btn-link btn--sm'
                },
                'Today'
            )
        );
    };

    return DatePickerPopup;
}(_react2.default.Component);

DatePickerPopup.propTypes = PROPERTY_TYPES;
DatePickerPopup.defaultProps = DEFAULT_PROPS;

exports.default = DatePickerPopup;