'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _CSSTransitionGroup = require('react-transition-group/CSSTransitionGroup');

var _CSSTransitionGroup2 = _interopRequireDefault(_CSSTransitionGroup);

var _dateUtils = require('../util/date-utils');

var _dateFormats = require('../util/date-formats');

var _utils = require('../util/utils');

var _DatePickerCalendar = require('../DatePicker/DatePickerCalendar');

var _DatePickerCalendar2 = _interopRequireDefault(_DatePickerCalendar);

var _Popup = require('../Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _IconInput = require('../InputIcon/IconInput');

var _IconInput2 = _interopRequireDefault(_IconInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    format: _propTypes2.default.func,
    parse: _propTypes2.default.func,
    formatHeader: _propTypes2.default.func,
    open: _propTypes2.default.bool,
    onPopupStateChange: _propTypes2.default.func,
    value: _propTypes2.default.shape({
        from: _propTypes2.default.string,
        to: _propTypes2.default.string
    })
};
var DEFAULT_PROPS = {
    format: _dateFormats.formatDDMMYYYY,
    parse: _dateFormats.parseDDMMYYYY,
    formatHeader: _dateFormats.formatFullMonthYear,
    value: {
        from: '',
        to: ''
    }
};

var DatePicker = function (_React$Component) {
    _inherits(DatePicker, _React$Component);

    function DatePicker(props) {
        _classCallCheck(this, DatePicker);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            month: (0, _dateUtils.getToday)(),
            over: null,
            animation: 'date-range-picker__calendar--animation-right',
            open: false
        };

        _this.handleSelection = _this.handleSelection.bind(_this);
        _this.handleMouseEnter = _this.handleMouseEnter.bind(_this);
        _this.handleMouseLeave = _this.handleMouseLeave.bind(_this);
        _this.getDateStyles = _this.getDateStyles.bind(_this);
        _this.transitioning = false;
        return _this;
    }

    DatePicker.prototype.componentDidMount = function componentDidMount() {
        this.setState({
            from: this.props.value.from ? this.parseDate(this.props.value.from) : null,
            fromText: this.props.value.from ? this.props.value.from : '',
            to: this.props.value.to ? this.parseDate(this.props.value.to) : null,
            toText: this.props.value.to ? this.props.value.to : ''
        });
    };

    DatePicker.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (nextProps.value) {
            var from = nextProps.value.from;
            var to = nextProps.value.to;
            this.setState({
                from: this.parseDate(from),
                fromText: from,
                to: this.parseDate(to),
                toText: to
            });
        }

        if ((0, _utils.isDefined)(nextProps.open)) {
            this.setState({
                open: nextProps.open
            });
        }
    };

    DatePicker.prototype.isPopupControlled = function isPopupControlled() {
        return (0, _utils.isDefined)(this.props.open);
    };

    DatePicker.prototype.isControlled = function isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    DatePicker.prototype.isOpen = function isOpen() {
        if (this.isPopupControlled()) {
            return this.props.open;
        }

        return this.state.open;
    };

    DatePicker.prototype.left = function left() {
        if (this.transitioning) {
            return;
        }
        this.transitioning = true;

        this.setState({
            month: (0, _dateUtils.addMonths)(this.state.month, -1),
            animation: 'date-range-picker__calendar--animation-left'
        });
    };

    DatePicker.prototype.right = function right() {
        if (this.transitioning) {
            return;
        }
        this.transitioning = true;

        this.setState({
            month: (0, _dateUtils.addMonths)(this.state.month, 1),
            animation: 'date-range-picker__calendar--animation-right'
        });
    };

    DatePicker.prototype.getDateStyles = function getDateStyles(date) {
        var isFrom = (0, _dateUtils.isDayTheSame)(date, this.state.from);
        var isTo = (0, _dateUtils.isDayTheSame)(date, this.state.to);
        var isBetween = (0, _dateUtils.isBetweenNotInclusive)(date, this.state.from, this.state.to || this.state.over);
        var isHovered = this.state.over && (0, _dateUtils.isDayTheSame)(date, this.state.over);
        var isFromOnHover = !isTo && !isBetween && isHovered && this.isFromDateWillBePlaced(this.state.over);
        var isToOnHover = !isBetween && isHovered && !isFromOnHover;

        return (0, _classnames2.default)({
            'date-range-picker__cell--between': isBetween && !isHovered,
            'date-range-picker__cell--from': isFrom || isFromOnHover,
            'date-range-picker__cell--to': isTo || isToOnHover,
            'date-range-picker__cell--hover-square': isBetween && isHovered
        });
    };

    DatePicker.prototype.handleSelection = function handleSelection(selected) {
        if (this.isFromDateWillBePlaced(selected)) {
            this.updateStateFromTo(selected, this.formatDate(selected), null, '');
        } else {
            this.updateStateTo(selected, this.formatDate(selected));
        }
    };

    DatePicker.prototype.isFromDateWillBePlaced = function isFromDateWillBePlaced(selected) {
        return !this.state.from || this.state.from && this.state.to || (0, _dateUtils.isAfter)(this.state.from, selected);
    };

    DatePicker.prototype.handleMouseEnter = function handleMouseEnter(selected) {
        this.setState({
            over: selected
        });
    };

    DatePicker.prototype.handleMouseLeave = function handleMouseLeave() {
        this.setState({
            over: null
        });
    };

    DatePicker.prototype.updateFrom = function updateFrom(value) {
        this.updateStateFrom(this.parseDate(value), value);
    };

    DatePicker.prototype.updateTo = function updateTo(value) {
        this.updateStateTo(this.parseDate(value), value);
    };

    DatePicker.prototype.updateStateFromTo = function updateStateFromTo(from, fromText, to, toText) {
        if (this.isControlled()) {
            this.fireChangeEvent(fromText, toText);
        } else {
            this.setState({
                from: from,
                fromText: fromText,
                to: to,
                toText: toText
            }, this.fireChangeEventByState());
        }
    };

    DatePicker.prototype.updateStateTo = function updateStateTo(to, toText) {
        var open = this.state.from === null || to === null;
        if (this.isControlled()) {
            this.fireChangeEvent(this.state.fromText, toText, open);
        } else {
            this.setState({
                to: to,
                toText: toText,
                open: open
            }, this.fireChangeEventByState);
        }
    };

    DatePicker.prototype.updateStateFrom = function updateStateFrom(from, fromText) {
        var open = this.state.to === null || from === null;
        if (this.isControlled()) {
            this.fireChangeEvent(fromText, this.state.toText, open);
        } else {
            this.setState({
                from: from,
                fromText: fromText,
                open: open
            }, this.fireChangeEventByState);
        }
    };

    DatePicker.prototype.fireChangeEventByState = function fireChangeEventByState() {
        this.fireChangeEvent(this.state.fromText, this.state.toText, this.state.open);
    };

    DatePicker.prototype.fireChangeEvent = function fireChangeEvent(fromText, toText) {
        var open = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

        if (this.props.onChange) {
            this.props.onChange({
                from: fromText,
                to: toText
            });
        }
        this.changePopupState(open);
    };

    DatePicker.prototype.changePopupState = function changePopupState(open) {
        if (!this.isPopupControlled()) {
            this.setState({ open: open });
        }
        if (this.props.onPopupStateChange) {
            this.props.onPopupStateChange(open);
        }
    };

    DatePicker.prototype.formatDate = function formatDate(date) {
        if (date) {
            return this.props.format(date);
        }
        return null;
    };

    DatePicker.prototype.parseDate = function parseDate(date) {
        var parsed = this.props.parse(date);
        if (parsed.isValid()) {
            return parsed;
        } else {
            return null;
        }
    };

    DatePicker.prototype.resetMonth = function resetMonth() {
        this.setState({
            month: this.state.from || this.state.to || (0, _dateUtils.getToday)()
        });
    };

    DatePicker.prototype.renderCalendars = function renderCalendars() {
        var left = this.state.month;
        var right = (0, _dateUtils.addMonths)(left, 1);
        var calendars = [];
        calendars.push(this.renderCalendar(left));
        calendars.push(this.renderCalendar(right));
        return calendars;
    };

    DatePicker.prototype.renderCalendar = function renderCalendar(date) {
        var _this2 = this;

        var formattedDate = this.props.formatHeader(date);
        return _react2.default.createElement(
            'div',
            { className: 'date-range-picker__calendar', key: formattedDate },
            _react2.default.createElement(
                'div',
                { className: 'date-range-picker__header' },
                formattedDate
            ),
            _react2.default.createElement(_DatePickerCalendar2.default, {
                onSelected: this.handleSelection,
                onMouseEnter: this.handleMouseEnter,
                onMouseLeave: this.handleMouseLeave,
                getStyles: this.getDateStyles,
                componentWillUnmount: function componentWillUnmount() {
                    _this2.transitioning = false;
                },
                month: date,
                startDateType: 'simple',
                renderDatesOfOtherMonth: false
            })
        );
    };

    DatePicker.prototype.render = function render() {
        var _this3 = this;

        var drpClassNames = (0, _classnames2.default)(this.props.className, 'date-range-picker-control');
        return _react2.default.createElement(
            _Popup2.default,
            {
                popupClassName: 'popover-colored',
                attachment: 'bottom left',
                open: this.isOpen(),
                onPopupStateChange: function onPopupStateChange(open) {
                    return _this3.changePopupState(open);
                },
                on: 'focus',
                onTransitionClosedToOpen: function onTransitionClosedToOpen() {
                    _this3.resetMonth();
                }
            },
            _react2.default.createElement(
                'div',
                { className: drpClassNames },
                _react2.default.createElement(_IconInput2.default, {
                    value: this.state.fromText,
                    onChange: function onChange(value) {
                        return _this3.updateFrom(value);
                    },
                    className: 'date-range-picker-control__input',
                    position: 'right',
                    iconClassName: 'im icon-calendar-from'
                }),
                _react2.default.createElement(_IconInput2.default, {
                    value: this.state.toText,
                    onChange: function onChange(value) {
                        return _this3.updateTo(value);
                    },
                    className: 'date-range-picker-control__input',
                    position: 'right',
                    iconClassName: 'im icon-calendar-to'
                })
            ),
            _react2.default.createElement(
                'div',
                { tabIndex: '-1', className: 'date-range-picker' },
                _react2.default.createElement('i', { onClick: function onClick() {
                        return _this3.left();
                    }, className: 'im icon-arrow-thin-left date-range-picker__arrow' }),
                _react2.default.createElement(
                    _CSSTransitionGroup2.default,
                    {
                        transitionEnter: true,
                        transitionLeave: true,
                        transitionEnterTimeout: 300,
                        transitionLeaveTimeout: 300,
                        transitionName: this.state.animation,
                        className: 'date-range-picker__calendars'
                    },
                    this.renderCalendars()
                ),
                _react2.default.createElement('i', { onClick: function onClick() {
                        return _this3.right();
                    }, className: 'im icon-arrow-thin-right date-range-picker__arrow' })
            )
        );
    };

    return DatePicker;
}(_react2.default.Component);

DatePicker.propTypes = PROPERTY_TYPES;
DatePicker.defaultProps = DEFAULT_PROPS;

exports.default = DatePicker;