'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _messages = require('./utils/messages');

var _messages2 = _interopRequireDefault(_messages);

var _localizer = require('./localizer');

var _localizer2 = _interopRequireDefault(_localizer);

var _dates = require('./utils/dates');

var _dates2 = _interopRequireDefault(_dates);

var _constants = require('./utils/constants');

var _accessors = require('./utils/accessors');

var _class = require('dom-helpers/class');

var _class2 = _interopRequireDefault(_class);

var _width = require('dom-helpers/query/width');

var _width2 = _interopRequireDefault(_width);

var _scrollbarSize = require('dom-helpers/util/scrollbarSize');

var _scrollbarSize2 = _interopRequireDefault(_scrollbarSize);

var _eventLevels = require('./utils/eventLevels');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    messages: _propTypes2.default.shape({
        date: _propTypes2.default.string,
        time: _propTypes2.default.string,
        event: _propTypes2.default.string
    })
};

var DEFAULT_PROPS = {
    length: 30
};

var Agenda = function (_React$Component) {
    _inherits(Agenda, _React$Component);

    function Agenda() {
        _classCallCheck(this, Agenda);

        return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
    }

    Agenda.prototype.componentDidMount = function componentDidMount() {
        this._adjustHeader();
    };

    Agenda.prototype.componentDidUpdate = function componentDidUpdate() {
        this._adjustHeader();
    };

    Agenda.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            length = _props.length,
            date = _props.date,
            events = _props.events,
            startAccessor = _props.startAccessor;

        var messages = (0, _messages2.default)(this.props.messages);
        var end = _dates2.default.add(date, length, 'day');

        var range = _dates2.default.range(date, end, 'day');

        events = events.filter(function (event) {
            return (0, _eventLevels.inRange)(event, date, end, _this2.props);
        });

        events.sort(function (a, b) {
            return +(0, _accessors.accessor)(a, startAccessor) - +(0, _accessors.accessor)(b, startAccessor);
        });

        return _react2.default.createElement(
            'div',
            { className: 'rbc-agenda-view' },
            _react2.default.createElement(
                'table',
                { ref: 'header' },
                _react2.default.createElement(
                    'thead',
                    null,
                    _react2.default.createElement(
                        'tr',
                        null,
                        _react2.default.createElement(
                            'th',
                            { className: 'rbc-header', ref: 'dateCol' },
                            messages.date
                        ),
                        _react2.default.createElement(
                            'th',
                            { className: 'rbc-header', ref: 'timeCol' },
                            messages.time
                        ),
                        _react2.default.createElement(
                            'th',
                            { className: 'rbc-header' },
                            messages.event
                        )
                    )
                )
            ),
            _react2.default.createElement(
                'div',
                { className: 'rbc-agenda-content', ref: 'content' },
                _react2.default.createElement(
                    'table',
                    null,
                    _react2.default.createElement(
                        'tbody',
                        { ref: 'tbody' },
                        range.map(function (day, idx) {
                            return _this2.renderDay(day, events, idx);
                        })
                    )
                )
            )
        );
    };

    Agenda.prototype.renderDay = function renderDay(day, events, dayKey) {
        var _this3 = this;

        var _props2 = this.props,
            culture = _props2.culture,
            components = _props2.components,
            titleAccessor = _props2.titleAccessor,
            agendaDateFormat = _props2.agendaDateFormat;


        var EventComponent = components.event;
        var DateComponent = components.date;

        events = events.filter(function (e) {
            return (0, _eventLevels.inRange)(e, day, day, _this3.props);
        });

        return events.map(function (event, idx) {
            var dateLabel = idx === 0 && _localizer2.default.format(day, agendaDateFormat, culture);
            var first = idx === 0 ? _react2.default.createElement(
                'td',
                { rowSpan: events.length, className: 'rbc-agenda-date-cell' },
                DateComponent ? _react2.default.createElement(DateComponent, { day: day, label: dateLabel }) : dateLabel
            ) : false;

            var title = (0, _accessors.accessor)(event, titleAccessor);

            return _react2.default.createElement(
                'tr',
                { key: dayKey + '_' + idx },
                first,
                _react2.default.createElement(
                    'td',
                    { className: 'rbc-agenda-time-cell' },
                    _this3.timeRangeLabel(day, event)
                ),
                _react2.default.createElement(
                    'td',
                    { className: 'rbc-agenda-event-cell' },
                    EventComponent ? _react2.default.createElement(EventComponent, { event: event, title: title }) : title
                )
            );
        }, []);
    };

    Agenda.prototype.timeRangeLabel = function timeRangeLabel(day, event) {
        var _props3 = this.props,
            endAccessor = _props3.endAccessor,
            startAccessor = _props3.startAccessor,
            allDayAccessor = _props3.allDayAccessor,
            culture = _props3.culture,
            messages = _props3.messages,
            components = _props3.components;


        var labelClass = '',
            TimeComponent = components.time,
            label = (0, _messages2.default)(messages).allDay;

        var start = (0, _accessors.accessor)(event, startAccessor);
        var end = (0, _accessors.accessor)(event, endAccessor);

        if (!(0, _accessors.accessor)(event, allDayAccessor)) {
            if (_dates2.default.eq(start, end, 'day')) {
                label = _localizer2.default.format({ start: start, end: end }, this.props.agendaTimeRangeFormat, culture);
            } else if (_dates2.default.eq(day, start, 'day')) {
                label = _localizer2.default.format(start, this.props.agendaTimeFormat, culture);
            } else if (_dates2.default.eq(day, end, 'day')) {
                label = _localizer2.default.format(start, this.props.agendaTimeFormat, culture);
            }
        }

        if (_dates2.default.gt(day, start, 'day')) labelClass = 'rbc-continues-prior';
        if (_dates2.default.lt(day, end, 'day')) labelClass += ' rbc-continues-after';

        return _react2.default.createElement(
            'span',
            { className: labelClass.trim() },
            TimeComponent ? _react2.default.createElement(TimeComponent, { event: event, label: label }) : label
        );
    };

    Agenda.prototype._adjustHeader = function _adjustHeader() {
        var header = this.refs.header;
        var firstRow = this.refs.tbody.firstChild;

        if (!firstRow) return;

        var isOverflowing = this.refs.content.scrollHeight > this.refs.content.clientHeight;
        var widths = this._widths || [];

        this._widths = [(0, _width2.default)(firstRow.children[0]), (0, _width2.default)(firstRow.children[1])];

        if (widths[0] !== this._widths[0] || widths[1] !== this._widths[1]) {
            this.refs.dateCol.style.width = this._widths[0] + 'px';
            this.refs.timeCol.style.width = this._widths[1] + 'px';
        }

        if (isOverflowing) {
            _class2.default.addClass(header, 'rbc-header-overflowing');
            header.style.marginRight = (0, _scrollbarSize2.default)() + 'px';
        } else {
            _class2.default.removeClass(header, 'rbc-header-overflowing');
        }
    };

    return Agenda;
}(_react2.default.Component);

Agenda.navigate = function (date, action) {
    switch (action) {
        case _constants.navigate.PREVIOUS:
            return _dates2.default.add(date, -1, 'day');

        case _constants.navigate.NEXT:
            return _dates2.default.add(date, 1, 'day');

        default:
            return date;
    }
};

Agenda.range = function (start, _ref) {
    var _ref$length = _ref.length,
        length = _ref$length === undefined ? Agenda.defaultProps.length : _ref$length;

    var end = _dates2.default.add(start, length, 'day');
    return { start: start, end: end };
};

Agenda.propTypes = PROPERTY_TYPES;
Agenda.defaultProps = DEFAULT_PROPS;

exports.default = Agenda;