'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _eventLevels = require('./utils/eventLevels');

var _helpers = require('./utils/helpers');

var _selection = require('./utils/selection');

var _Selection = require('./Selection');

var _Selection2 = _interopRequireDefault(_Selection);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    selectable: _propTypes2.default.bool,
    draggableSelect: _propTypes2.default.bool,
    onSelect: _propTypes2.default.func,
    slots: _propTypes2.default.number,
    highlight: _propTypes2.default.func,
    isHovered: _propTypes2.default.func,
    isEndDate: _propTypes2.default.func,
    isToday: _propTypes2.default.func,
    isStartDate: _propTypes2.default.func,
    onHover: _propTypes2.default.func
};

var DisplayCells = function (_React$Component) {
    _inherits(DisplayCells, _React$Component);

    function DisplayCells(props) {
        _classCallCheck(this, DisplayCells);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = { selecting: false };
        return _this;
    }

    DisplayCells.prototype.componentDidMount = function componentDidMount() {
        this.props.selectable && this._selectable();
        this.props.draggableSelect && this._draggable();
    };

    DisplayCells.prototype.componentWillUnmount = function componentWillUnmount() {
        this._teardownSelectable();
        this._teardownDraggable();
    };

    DisplayCells.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (nextProps.selectable && !this.props.selectable) this._selectable();
        if (!nextProps.selectable && this.props.selectable) this._teardownSelectable();

        if (nextProps.draggableSelect && !this.props.draggableSelect) this._draggable();
        if (!nextProps.draggableSelect && this.props.draggableSelect) this._teardownDraggable();
    };

    DisplayCells.prototype.render = function render() {
        var slots = this.props.slots;
        var _state = this.state,
            selecting = _state.selecting,
            startIdx = _state.startIdx,
            endIdx = _state.endIdx;


        var children = [];

        for (var i = 0; i < slots; i++) {
            children.push(_react2.default.createElement('div', {
                key: 'bg_' + i,
                style: (0, _eventLevels.segStyle)(1, slots),
                className: (0, _classnames2.default)('rbc-day-bg', {
                    'rbc-selected': this.props.isSelected(i),
                    'rbc-day-highlighted': this.props.highlight(i),
                    'rbc-end-date': this.props.isEndDate(i),
                    'rbc-hovered-date': this.props.isHovered(i),
                    'rbc-current-bg-cell': this.props.isToday(i),
                    'rbc-selected-cell': selecting && i >= startIdx && i <= endIdx
                })
            }));
        }

        return _react2.default.createElement(
            'div',
            { className: 'rbc-row-bg' },
            children
        );
    };

    DisplayCells.prototype._draggable = function _draggable() {
        var _this2 = this;

        var node = (0, _reactDom.findDOMNode)(this);
        var selector = this._dragableSelector = new _Selection2.default(this.props.container);
        selector.on('selecting', function (box) {
            var slots = _this2.props.slots;


            var startIdx = -1;
            var endIdx = -1;

            if (!_this2.state.selecting) {
                (0, _helpers.notify)(_this2.props.onSelectStart, [box]);
                _this2._initial = { x: box.x, y: box.y };
            }
            if (selector.isSelected(node)) {
                var nodeBox = (0, _Selection.getBoundsForNode)(node);

                var _dateCellSelection = (0, _selection.dateCellSelection)(_this2._initial, nodeBox, box, slots);

                startIdx = _dateCellSelection.startIdx;
                endIdx = _dateCellSelection.endIdx;
            }

            _this2.setState({
                selecting: true,
                startIdx: startIdx,
                endIdx: endIdx
            });
        });
        selector.on('select', function (event) {
            _this2._selectSlot(_this2.state);
            _this2._initial = {};
            _this2.setState({ selecting: false });
            (0, _helpers.notify)(_this2.props.onSelectEnd, [_this2.state]);
        });
    };

    DisplayCells.prototype._selectable = function _selectable() {
        var _this3 = this;

        var node = (0, _reactDom.findDOMNode)(this);
        var selector = this._selector = new _Selection2.default(this.props.container);
        selector.on('click', function (point, event) {
            if (event.target.closest('.rbc-event') || event.target.closest('.rbc-show-more')) {
                return;
            }
            var rowBox = (0, _Selection.getBoundsForNode)(node);

            if ((0, _selection.pointInBox)(rowBox, point)) {
                var width = (0, _selection.slotWidth)((0, _Selection.getBoundsForNode)(node), _this3.props.slots);
                var currentCell = (0, _selection.getCellAtX)(rowBox, point.x, width);

                _this3._selectSlot({
                    startIdx: currentCell,
                    endIdx: currentCell,
                    selectedIndex: currentCell
                }, event);
            }

            _this3._initial = {};

            _this3.setState({
                selecting: false
            });
        });
        selector.on('mouseover', function (point) {
            var node = (0, _reactDom.findDOMNode)(_this3);
            var rowBox = (0, _Selection.getBoundsForNode)(node);
            if ((0, _selection.pointInBox)(rowBox, point)) {
                var width = (0, _selection.slotWidth)((0, _Selection.getBoundsForNode)(node), _this3.props.slots);
                var currentCell = (0, _selection.getCellAtX)(rowBox, point.x, width);
                _this3.props.onHover(currentCell);
            }
        });
    };

    DisplayCells.prototype._teardownSelectable = function _teardownSelectable() {
        if (!this._selector) return;
        this._selector.teardown();
        this._selector = null;
    };

    DisplayCells.prototype._teardownDraggable = function _teardownDraggable() {
        if (!this._dragableSelector) return;
        this._dragableSelector.teardown();
        this._dragableSelector = null;
    };

    DisplayCells.prototype._selectSlot = function _selectSlot(_ref, event) {
        var endIdx = _ref.endIdx,
            startIdx = _ref.startIdx,
            selectedIndex = _ref.selectedIndex;

        this.props.onSelectSlot && this.props.onSelectSlot({
            start: startIdx,
            end: endIdx,
            selectedIndex: selectedIndex
        }, event);
    };

    return DisplayCells;
}(_react2.default.Component);

DisplayCells.propTypes = PROPERTY_TYPES;

exports.default = DisplayCells;