'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Selection = require('./Selection');

var _Selection2 = _interopRequireDefault(_Selection);

var _dates = require('./utils/dates');

var _dates2 = _interopRequireDefault(_dates);

var _selection = require('./utils/selection');

var _localizer = require('./localizer');

var _localizer2 = _interopRequireDefault(_localizer);

var _helpers = require('./utils/helpers');

var _propTypes3 = require('./utils/propTypes');

var _accessors = require('./utils/accessors');

var _TimeColumn = require('./TimeColumn');

var _TimeColumn2 = _interopRequireDefault(_TimeColumn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function snapToSlot(date, step) {
    var roundTo = 1000 * 60 * step;
    return new Date(Math.floor(date.getTime() / roundTo) * roundTo);
}

function positionFromDate(date, min) {
    return _dates2.default.diff(min, _dates2.default.merge(min, date), 'minutes');
}

function overlaps(event, events, _ref, last) {
    var startAccessor = _ref.startAccessor,
        endAccessor = _ref.endAccessor;

    var eStart = (0, _accessors.accessor)(event, startAccessor);
    var offset = last;

    function overlap(eventB) {
        return _dates2.default.lt(eStart, (0, _accessors.accessor)(eventB, endAccessor));
    }

    if (!events.length) return last - 1;
    events.reverse().some(function (prevEvent) {
        //eslint-disable-line
        if (overlap(prevEvent)) return true;
        offset = offset - 1;
    });

    return offset;
}

var PROPERTY_TYPES = {
    events: _propTypes2.default.array.isRequired,
    step: _propTypes2.default.number.isRequired,
    min: _propTypes2.default.instanceOf(Date).isRequired,
    max: _propTypes2.default.instanceOf(Date).isRequired,

    allDayAccessor: _propTypes3.accessor.isRequired,
    startAccessor: _propTypes3.accessor.isRequired,
    endAccessor: _propTypes3.accessor.isRequired,

    selectable: _propTypes2.default.bool,
    eventOffset: _propTypes2.default.number,

    onSelecting: _propTypes2.default.func,
    onSelectSlot: _propTypes2.default.func.isRequired,
    onSelectEvent: _propTypes2.default.func.isRequired,

    className: _propTypes2.default.string
};

var DaySlot = function (_React$Component) {
    _inherits(DaySlot, _React$Component);

    function DaySlot(props) {
        _classCallCheck(this, DaySlot);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = { selecting: false };
        _this._clickTimer = null;
        return _this;
    }

    DaySlot.prototype.componentDidMount = function componentDidMount() {
        this.props.selectable && this._selectable();
    };

    DaySlot.prototype.componentWillUnmount = function componentWillUnmount() {
        this._teardownSelectable();
    };

    DaySlot.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (nextProps.selectable && !this.props.selectable) this._selectable();
        if (!nextProps.selectable && this.props.selectable) this._teardownSelectable();
    };

    DaySlot.prototype.renderEvents = function renderEvents() {
        var _this2 = this;

        var _props = this.props,
            events = _props.events,
            step = _props.step,
            min = _props.min,
            culture = _props.culture,
            eventPropGetter = _props.eventPropGetter,
            selected = _props.selected,
            eventTimeRangeFormat = _props.eventTimeRangeFormat,
            eventComponent = _props.eventComponent,
            startAccessor = _props.startAccessor,
            endAccessor = _props.endAccessor,
            titleAccessor = _props.titleAccessor;


        var EventComponent = eventComponent,
            lastLeftOffset = 0;

        events.sort(function (a, b) {
            return +(0, _accessors.accessor)(a, startAccessor) - +(0, _accessors.accessor)(b, startAccessor);
        });

        return events.map(function (event, idx) {
            var start = (0, _accessors.accessor)(event, startAccessor);
            var end = (0, _accessors.accessor)(event, endAccessor);
            var startSlot = positionFromDate(start, min, step);
            var endSlot = positionFromDate(end, min, step);

            lastLeftOffset = Math.max(0, overlaps(event, events.slice(0, idx), _this2.props, lastLeftOffset + 1));

            var style = _this2._slotStyle(startSlot, endSlot, lastLeftOffset);

            var title = (0, _accessors.accessor)(event, titleAccessor);
            var label = _localizer2.default.format({ start: start, end: end }, eventTimeRangeFormat, culture);
            var _isSelected = (0, _selection.isSelected)(event, selected);

            if (eventPropGetter) var _eventPropGetter = eventPropGetter(event, start, end, _isSelected),
                    xStyle = _eventPropGetter.style,
                    className = _eventPropGetter.className; //eslint-disable-line

            return _react2.default.createElement(
                'div',
                {
                    key: 'evt_' + idx,
                    style: _extends({}, xStyle, style),
                    title: label + ': ' + title,
                    onClick: _this2._select.bind(null, event),
                    className: (0, _classnames2.default)('rbc-event', className, {
                        'rbc-selected': _isSelected,
                        'rbc-event-overlaps': lastLeftOffset !== 0
                    })
                },
                _react2.default.createElement(
                    'div',
                    { className: 'rbc-event-label' },
                    label
                ),
                _react2.default.createElement(
                    'div',
                    { className: 'rbc-event-content' },
                    EventComponent ? _react2.default.createElement(EventComponent, { event: event, title: title }) : title
                )
            );
        });
    };

    DaySlot.prototype._slotStyle = function _slotStyle(startSlot, endSlot, leftOffset) {
        var _ref2;

        endSlot = Math.max(endSlot, startSlot + this.props.step); //must be at least one `step` high

        var eventOffset = this.props.eventOffset || 10,
            isRtl = this.props.rtl;

        var top = startSlot / this._totalMin * 100;
        var bottom = endSlot / this._totalMin * 100;
        var per = leftOffset === 0 ? 0 : leftOffset * eventOffset;
        var rightDiff = eventOffset / (leftOffset + 1);

        return _ref2 = {
            top: top + '%',
            height: bottom - top + '%'
        }, _defineProperty(_ref2, isRtl ? 'right' : 'left', per + '%'), _defineProperty(_ref2, 'width', (leftOffset === 0 ? 100 - eventOffset : 100 - per - rightDiff) + '%'), _ref2;
    };

    DaySlot.prototype._selectable = function _selectable() {
        var _this3 = this;

        var node = (0, _reactDom.findDOMNode)(this);
        var selector = this._selector = new _Selection2.default(function () {
            return (0, _reactDom.findDOMNode)(_this3);
        });

        var maybeSelect = function maybeSelect(box) {
            var onSelecting = _this3.props.onSelecting;
            var current = _this3.state || {};
            var state = selectionState(box);
            var start = state.startDate,
                end = state.endDate;


            if (onSelecting) {
                if (_dates2.default.eq(current.startDate, start, 'minutes') && _dates2.default.eq(current.endDate, end, 'minutes') || onSelecting({ start: start, end: end }) === false) return;
            }

            _this3.setState(state);
        };

        var selectionState = function selectionState(_ref3) {
            var y = _ref3.y;
            var _props2 = _this3.props,
                step = _props2.step,
                min = _props2.min,
                max = _props2.max;

            var _getBoundsForNode = (0, _Selection.getBoundsForNode)(node),
                top = _getBoundsForNode.top,
                bottom = _getBoundsForNode.bottom;

            var mins = _this3._totalMin;

            var range = Math.abs(top - bottom);

            var current = (y - top) / range;

            current = snapToSlot(minToDate(mins * current, min), step);

            if (!_this3.state.selecting) _this3._initialDateSlot = current;

            var initial = _this3._initialDateSlot;

            if (_dates2.default.eq(initial, current, 'minutes')) current = _dates2.default.add(current, step, 'minutes');

            var start = _dates2.default.max(min, _dates2.default.min(initial, current));
            var end = _dates2.default.min(max, _dates2.default.max(initial, current));

            return {
                selecting: true,
                startDate: start,
                endDate: end,
                startSlot: positionFromDate(start, min, step),
                endSlot: positionFromDate(end, min, step)
            };
        };

        selector.on('selecting', maybeSelect);
        selector.on('selectStart', maybeSelect);

        selector.on('click', function (_ref4) {
            var x = _ref4.x,
                y = _ref4.y;

            _this3._clickTimer = setTimeout(function () {
                _this3._selectSlot(selectionState({ x: x, y: y }));
            });

            _this3.setState({ selecting: false });
        });

        selector.on('select', function () {
            if (_this3.state.selecting) {
                _this3._selectSlot(_this3.state);
                _this3.setState({ selecting: false });
            }
        });
    };

    DaySlot.prototype._teardownSelectable = function _teardownSelectable() {
        if (!this._selector) return;
        this._selector.teardown();
        this._selector = null;
    };

    DaySlot.prototype._selectSlot = function _selectSlot(_ref5) {
        var startDate = _ref5.startDate,
            endDate = _ref5.endDate;

        var current = startDate,
            slots = [];

        while (_dates2.default.lte(current, endDate)) {
            slots.push(current);
            current = _dates2.default.add(current, this.props.step, 'minutes');
        }

        (0, _helpers.notify)(this.props.onSelectSlot, {
            slots: slots,
            start: startDate,
            end: endDate
        });
    };

    DaySlot.prototype._select = function _select(event) {
        clearTimeout(this._clickTimer);
        (0, _helpers.notify)(this.props.onSelectEvent, event);
    };

    DaySlot.prototype.render = function render() {
        var _props3 = this.props,
            min = _props3.min,
            max = _props3.max,
            step = _props3.step,
            timeslots = _props3.timeslots,
            now = _props3.now,
            selectRangeFormat = _props3.selectRangeFormat,
            culture = _props3.culture,
            props = _objectWithoutProperties(_props3, ['min', 'max', 'step', 'timeslots', 'now', 'selectRangeFormat', 'culture']);

        this._totalMin = _dates2.default.diff(min, max, 'minutes');

        var _state = this.state,
            selecting = _state.selecting,
            startSlot = _state.startSlot,
            endSlot = _state.endSlot,
            style = this._slotStyle(startSlot, endSlot, 0);

        var selectDates = {
            start: this.state.startDate,
            end: this.state.endDate
        };

        return _react2.default.createElement(
            _TimeColumn2.default,
            _extends({}, props, {
                className: 'rbc-day-slot',
                timeslots: timeslots,
                now: now,
                min: min,
                max: max,
                step: step
            }),
            this.renderEvents(),
            selecting && _react2.default.createElement(
                'div',
                { className: 'rbc-slot-selection', style: style },
                _react2.default.createElement(
                    'span',
                    null,
                    _localizer2.default.format(selectDates, selectRangeFormat, culture)
                )
            )
        );
    };

    return DaySlot;
}(_react2.default.Component);

function minToDate(min, date) {
    var dt = new Date(date),
        totalMins = _dates2.default.diff(_dates2.default.startOf(date, 'day'), date, 'minutes');

    dt = _dates2.default.hours(dt, 0);
    dt = _dates2.default.minutes(dt, totalMins + min);
    dt = _dates2.default.seconds(dt, 0);
    return _dates2.default.milliseconds(dt, 0);
}

DaySlot.propTypes = PROPERTY_TYPES;

exports.default = DaySlot;