'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _EventRowHOC = require('./EventRowHOC');

var _eventLevels = require('./utils/eventLevels');

var _messages = require('./utils/messages');

var _messages2 = _interopRequireDefault(_messages);

var _range = require('lodash/range');

var _range2 = _interopRequireDefault(_range);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isSegmentInSlot = function isSegmentInSlot(seg, slot) {
    return seg.left <= slot && seg.right >= slot;
};
var eventsInSlot = function eventsInSlot(segments, slot) {
    return segments.filter(function (seg) {
        return isSegmentInSlot(seg, slot);
    }).length;
};

var PROPERTY_TYPES = {
    segments: _propTypes2.default.array,
    slots: _propTypes2.default.number
};

var EventRow = function (_React$Component) {
    _inherits(EventRow, _React$Component);

    function EventRow(props) {
        _classCallCheck(this, EventRow);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.canRenderSlotEvent = _this.canRenderSlotEvent.bind(_this);
        _this.renderShowMore = _this.renderShowMore.bind(_this);
        _this._showMore = _this._showMore.bind(_this);
        return _this;
    }

    EventRow.prototype.render = function render() {
        var _props = this.props,
            segments = _props.segments,
            slotCount = _props.slots;

        var rowSegments = (0, _eventLevels.eventLevels)(segments).levels[0];

        var current = 1,
            lastEnd = 1,
            row = [];

        while (current <= slotCount) {
            var _ref = rowSegments.filter(function (seg) {
                return isSegmentInSlot(seg, current);
            })[0] || {},
                event = _ref.event,
                left = _ref.left,
                right = _ref.right,
                span = _ref.span; //eslint-disable-line

            var key = this.getLvlKey(current, event);

            if (!event) {
                current++;
                continue;
            }

            var gap = Math.max(0, left - lastEnd);

            if (this.canRenderSlotEvent(left, span)) {
                var content = this.renderEvent(event);

                if (gap) row.push(this.renderSpan(gap, key + '_gap'));

                row.push(this.renderSpan(span, key, content));

                lastEnd = current = right + 1;
            } else {
                if (gap) row.push(this.renderSpan(gap, key + '_gap'));

                row.push(this.renderSpan(1, key, this.renderShowMore(segments, current)));
                lastEnd = current = current + 1;
            }
        }

        return _react2.default.createElement(
            'div',
            { className: 'rbc-row' },
            row
        );
    };

    EventRow.prototype.canRenderSlotEvent = function canRenderSlotEvent(slot, span) {
        var segments = this.props.segments;


        return (0, _range2.default)(slot, slot + span).every(function (s) {
            var count = eventsInSlot(segments, s);

            return count === 1;
        });
    };

    EventRow.prototype.renderShowMore = function renderShowMore(segments, slot) {
        var messages = (0, _messages2.default)(this.props.messages);
        var count = eventsInSlot(segments, slot);

        return count ? _react2.default.createElement(
            'a',
            { key: 'sm_' + slot, href: '#', className: 'rbc-show-more', onClick: this._showMore.bind(null, slot) },
            messages.showMore(count)
        ) : false;
    };

    EventRow.prototype._showMore = function _showMore(slot, e) {
        e.preventDefault();
        this.props.onShowMore(slot);
    };

    return EventRow;
}(_react2.default.Component);

EventRow.propTypes = PROPERTY_TYPES;

exports.default = (0, _EventRowHOC.enhanceRowHOC)(EventRow);