'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _dates2 = require('./utils/dates');

var _dates3 = _interopRequireDefault(_dates2);

var _localizer = require('./localizer');

var _localizer2 = _interopRequireDefault(_localizer);

var _chunk = require('lodash/chunk');

var _chunk2 = _interopRequireDefault(_chunk);

var _collection = require('lodash/collection');

var _collection2 = _interopRequireDefault(_collection);

var _constants = require('./utils/constants');

var _helpers = require('./utils/helpers');

var _height = require('dom-helpers/query/height');

var _height2 = _interopRequireDefault(_height);

var _position = require('dom-helpers/query/position');

var _position2 = _interopRequireDefault(_position);

var _requestAnimationFrame = require('dom-helpers/util/requestAnimationFrame');

var _requestAnimationFrame2 = _interopRequireDefault(_requestAnimationFrame);

var _dateUtils = require('../util/date-utils');

var _EventRow = require('./EventRow');

var _EventRow2 = _interopRequireDefault(_EventRow);

var _EventEndingRow = require('./EventEndingRow');

var _EventEndingRow2 = _interopRequireDefault(_EventEndingRow);

var _Popup = require('./Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _Overlay = require('react-overlays/lib/Overlay');

var _Overlay2 = _interopRequireDefault(_Overlay);

var _BackgroundCells = require('./BackgroundCells');

var _BackgroundCells2 = _interopRequireDefault(_BackgroundCells);

var _propTypes3 = require('./utils/propTypes');

var _eventLevels2 = require('./utils/eventLevels');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var eventsForWeek = function eventsForWeek(evts, start, end, props) {
    return evts.filter(function (e) {
        return (0, _eventLevels2.inRange)(e, start, end, props);
    });
};

var isSegmentInSlot = function isSegmentInSlot(seg, slot) {
    return seg.left <= slot && seg.right >= slot;
};

var PROPERTY_TYPES = {
    culture: _propTypes2.default.string,

    date: _propTypes2.default.instanceOf(Date),

    min: _propTypes2.default.instanceOf(Date),
    max: _propTypes2.default.instanceOf(Date),

    dateFormat: _propTypes3.dateFormat,

    weekdayFormat: _propTypes3.dateFormat,

    popup: _propTypes2.default.bool,

    popupOffset: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.shape({
        x: _propTypes2.default.number,
        y: _propTypes2.default.number
    })]),

    onSelectEvent: _propTypes2.default.func,
    onSelectSlot: _propTypes2.default.func
};

var MonthView = function (_React$Component) {
    _inherits(MonthView, _React$Component);

    function MonthView(props) {
        _classCallCheck(this, MonthView);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            rowLimit: 5,
            needLimitMeasure: true,
            startDate: null,
            endDate: null,
            dropSelection: false
        };

        _this.renderWeek = _this.renderWeek.bind(_this);
        _this.renderBackground = _this.renderBackground.bind(_this);
        _this.renderRowLevel = _this.renderRowLevel.bind(_this);
        _this.renderShowMore = _this.renderShowMore.bind(_this);
        _this._dates = _this._dates.bind(_this);
        _this._headers = _this._headers.bind(_this);
        _this._renderMeasureRows = _this._renderMeasureRows.bind(_this);
        _this._renderOverlay = _this._renderOverlay.bind(_this);
        _this._measureRowLimit = _this._measureRowLimit.bind(_this);
        _this._dateClick = _this._dateClick.bind(_this);
        _this._selectEvent = _this._selectEvent.bind(_this);
        _this._selectDates = _this._selectDates.bind(_this);
        _this._showMore = _this._showMore.bind(_this);
        _this.clearSelection = _this.clearSelection.bind(_this);
        return _this;
    }

    MonthView.prototype.componentWillMount = function componentWillMount() {
        this._bgRows = [];
        this._pendingSelection = [];
    };

    MonthView.prototype.componentWillReceiveProps = function componentWillReceiveProps(_ref) {
        var date = _ref.date;

        this.setState({
            needLimitMeasure: !_dates3.default.eq(date, this.props.date)
        });
    };

    MonthView.prototype.componentDidMount = function componentDidMount() {
        var _this2 = this;

        var running = void 0;

        if (this.state.needLimitMeasure) this._measureRowLimit(this.props);

        window.addEventListener('click', this._dropSelection = function (e) {
            if (!e.target.closest('.rbc-calendar')) {
                _this2._reset();
            }
        });

        window.addEventListener('resize', this._resizeListener = function () {
            if (!running) {
                (0, _requestAnimationFrame2.default)(function () {
                    running = false;
                    _this2.setState({ needLimitMeasure: true }); //eslint-disable-line
                });
            }
        }, false);
    };

    MonthView.prototype.componentDidUpdate = function componentDidUpdate() {
        if (this.state.needLimitMeasure) this._measureRowLimit(this.props);
    };

    MonthView.prototype.componentWillUnmount = function componentWillUnmount() {
        window.removeEventListener('resize', this._resizeListener, false);
        window.removeEventListener('click', this._dropSelection, false);
    };

    MonthView.prototype.renderWeek = function renderWeek(week, weekIdx, content) {
        var _this3 = this;

        var _endOfRange = (0, _eventLevels2.endOfRange)(week),
            first = _endOfRange.first,
            last = _endOfRange.last;

        var evts = eventsForWeek(this.props.events, week[0], week[week.length - 1], this.props);
        var nonWorkingDays = this.props.nonWorkingDays;
        evts.sort(function (a, b) {
            return (0, _eventLevels2.sortEvents)(a, b, _this3.props);
        });

        var segments = evts = evts.map(function (evt) {
            return (0, _eventLevels2.eventSegments)(evt, first, last, _this3.props);
        });
        var limit = this.state.rowLimit - 1 || 1;

        var _eventLevels = (0, _eventLevels2.eventLevels)(segments, limit),
            levels = _eventLevels.levels,
            extra = _eventLevels.extra;

        content = content || function (lvls, wk) {
            return lvls.map(function (lvl, idx) {
                return _this3.renderRowLevel(lvl, wk, idx);
            });
        };

        return _react2.default.createElement(
            'div',
            { key: 'week_' + weekIdx, className: 'rbc-month-row', ref: !weekIdx && function (r) {
                    return _this3._firstRow = r;
                } },
            this.renderBackground(week, weekIdx),
            _react2.default.createElement(
                'div',
                { className: 'rbc-row-content' },
                _react2.default.createElement(
                    'div',
                    { className: 'rbc-row', ref: !weekIdx && function (r) {
                            return _this3._firstDateRow = r;
                        } },
                    this._dates(week, nonWorkingDays)
                ),
                content(levels, week, weekIdx),
                !!extra.length && this.renderShowMore(segments, extra, week, weekIdx, levels.length)
            )
        );
    };

    MonthView.prototype._setDate = function _setDate(date, event) {
        if (this._getStartDate()) {
            this.setState({
                endDate: date
            });
        } else {
            this.setState({
                startDate: date
            });
        }

        if (this._isStartEndDateSelected()) {
            var start = this._getStartDate();
            var end = this._getEndDate();

            if (!(0, _dateUtils.isDateBefore)(start, end)) {
                var temp = end;
                end = start;
                start = temp;
            }

            (0, _helpers.notify)(this.props.onRangeSelected, [{
                start: start,
                end: end
            }, event]);

            this._reset();
        }
    };

    MonthView.prototype._getStartDate = function _getStartDate() {
        return this.state.startDate && (0, _dateUtils.parse)(this.state.startDate);
    };

    MonthView.prototype._isStartEndDateSelected = function _isStartEndDateSelected() {
        return this._getStartDate() && this._getEndDate();
    };

    MonthView.prototype._getEndDate = function _getEndDate() {
        return this.state.endDate && (0, _dateUtils.parse)(this.state.endDate);
    };

    MonthView.prototype._getHoveredDate = function _getHoveredDate() {
        return this.state.hoveredDate && (0, _dateUtils.parse)(this.state.hoveredDate);
    };

    MonthView.prototype._reset = function _reset() {
        this.setState({
            endDate: null,
            startDate: null
        });
    };

    MonthView.prototype.renderBackground = function renderBackground(row, idx) {
        var _this4 = this;

        var self = this;

        function getCurrentDate(index) {
            var date = row.slice(index, index + 1)[0];
            return (0, _dateUtils.parse)(date);
        }

        function onSelectSlot(_ref2, event) {
            var start = _ref2.start,
                end = _ref2.end;

            var selectedDate = self._pendingSelection.concat(row.slice(start, end + 1));
            self._setDate(selectedDate[0], event);

            self._pendingSelection = selectedDate;

            clearTimeout(self._selectTimer);
            self._selectTimer = setTimeout(function () {
                return self._selectDates();
            });
        }

        function isEndDate(dateCell) {
            var date = getCurrentDate(dateCell);

            return (0, _dateUtils.isDatesSame)(date, self._hoveredDate) && self._getStartDate();
        }

        function isHovered(dateCell) {
            var date = getCurrentDate(dateCell);

            return (0, _dateUtils.isDatesSame)(date, self._hoveredDate) && !self._getStartDate();
        }

        function onHoverSlot(dateCell) {
            var date = getCurrentDate(dateCell);

            if ((0, _dateUtils.isDatesSame)(date, self._hoveredDate)) {
                return;
            }

            self._hoveredDate = date;
            self.setState({
                hoveredDate: date
            });
        }

        function highlight(cell) {
            var date = getCurrentDate(cell);
            var startDate = self._getStartDate();
            var hoveredDate = self._getHoveredDate();

            return hoveredDate && startDate && date && (0, _dateUtils.isDateBetween)(startDate, hoveredDate, date, '()') || (0, _dateUtils.isDateBetween)(hoveredDate, startDate, date, '()');
        }

        function isSelected(cell) {
            var startDate = self._getStartDate();
            var date = getCurrentDate(cell);

            return (0, _dateUtils.isDatesSame)(date, startDate);
        }

        function isToday(index) {
            var date = getCurrentDate(index);
            return _dates3.default.eq(_dates3.default.today(), date, 'day');
        }

        return _react2.default.createElement(_BackgroundCells2.default, {
            highlight: highlight,
            isSelected: isSelected,
            isHovered: isHovered,
            isEndDate: isEndDate,
            isToday: isToday,
            onHover: onHoverSlot,
            rowIndex: true,
            index: idx,
            container: function container() {
                return (0, _reactDom.findDOMNode)(_this4);
            },
            draggableSelect: this.props.draggableSelect,
            selectable: this.props.selectable,
            slots: 7,
            ref: function ref(r) {
                return _this4._bgRows[idx] = r;
            },
            onSelectSlot: onSelectSlot
        });
    };

    MonthView.prototype.renderRowLevel = function renderRowLevel(segments, week, idx) {
        var _endOfRange2 = (0, _eventLevels2.endOfRange)(week),
            first = _endOfRange2.first,
            last = _endOfRange2.last;

        return _react2.default.createElement(_EventRow2.default, _extends({}, this.props, {
            eventComponent: this.props.components.event,
            onSelect: this._selectEvent,
            key: idx,
            segments: segments,
            start: first,
            end: last
        }));
    };

    MonthView.prototype.renderShowMore = function renderShowMore(segments, extraSegments, week, weekIdx) {
        var _this5 = this;

        var _endOfRange3 = (0, _eventLevels2.endOfRange)(week),
            first = _endOfRange3.first,
            last = _endOfRange3.last;

        var onClick = function onClick(slot) {
            return _this5._showMore(segments, week[slot - 1], weekIdx, slot);
        };

        return _react2.default.createElement(_EventEndingRow2.default, _extends({}, this.props, {
            eventComponent: this.props.components.event,
            onSelect: this._selectEvent,
            onShowMore: onClick,
            key: 'last_row_' + weekIdx,
            segments: extraSegments,
            start: first,
            end: last
        }));
    };

    MonthView.prototype._dates = function _dates(row, nonWorkingDays) {
        var _this6 = this;

        return row.map(function (day, colIdx) {
            var offRange = _dates3.default.month(day) !== _dates3.default.month(_this6.props.date);
            var isNonWorkDay = _collection2.default.find(nonWorkingDays, function same(nonWorkDay) {
                return _dates3.default.isSameDate(nonWorkDay, day);
            });
            return _react2.default.createElement(
                'div',
                {
                    key: 'header_' + colIdx,
                    style: (0, _eventLevels2.segStyle)(1, 7),
                    className: (0, _classnames2.default)('rbc-date-cell', {
                        'rbc-off-range': offRange && !isNonWorkDay,
                        'rbc-off-range-non-work': offRange && isNonWorkDay,
                        'rbc-off-non-work': !offRange && isNonWorkDay,
                        'rbc-now': _dates3.default.eq(day, new Date(), 'day'),
                        'rbc-current': _dates3.default.eq(day, _this6.props.date, 'day')
                    })
                },
                _react2.default.createElement(
                    'a',
                    { href: '#', onClick: _this6._dateClick.bind(null, day) },
                    _localizer2.default.format(day, _this6.props.dateFormat, _this6.props.culture)
                )
            );
        });
    };

    MonthView.prototype._headers = function _headers(row, format, culture) {
        var first = row[0];
        var last = row[row.length - 1];

        return _dates3.default.range(first, last, 'day').map(function (day, idx) {
            return _react2.default.createElement(
                'div',
                { key: 'header_' + idx, className: 'rbc-header', style: (0, _eventLevels2.segStyle)(1, 7) },
                _localizer2.default.format(day, format, culture)
            );
        });
    };

    MonthView.prototype._renderMeasureRows = function _renderMeasureRows(levels, row, idx) {
        var _this7 = this;

        var first = idx === 0;

        return first ? _react2.default.createElement(
            'div',
            { className: 'rbc-row' },
            _react2.default.createElement(
                'div',
                { className: 'rbc-row-segment', style: (0, _eventLevels2.segStyle)(1, 7) },
                _react2.default.createElement(
                    'div',
                    { ref: function ref(r) {
                            return _this7._measureEvent = r;
                        }, className: (0, _classnames2.default)('rbc-event') },
                    _react2.default.createElement(
                        'div',
                        { className: 'rbc-event-content' },
                        '\xA0'
                    )
                )
            )
        ) : _react2.default.createElement('span', null);
    };

    MonthView.prototype._renderOverlay = function _renderOverlay() {
        var _this8 = this;

        var overlay = this.state && this.state.overlay || {};

        return _react2.default.createElement(
            _Overlay2.default,
            {
                rootClose: true,
                placement: 'bottom',
                container: this,
                show: !!overlay.position,
                onHide: function onHide() {
                    return _this8.setState({ overlay: null });
                }
            },
            _react2.default.createElement(_Popup2.default, _extends({}, this.props, {
                position: overlay.position,
                events: overlay.events,
                slotStart: overlay.date,
                slotEnd: overlay.end,
                onSelect: this._selectEvent
            }))
        );
    };

    MonthView.prototype._measureRowLimit = function _measureRowLimit() {
        var eventHeight = (0, _height2.default)(this._measureEvent);
        var labelHeight = (0, _height2.default)(this._firstDateRow);
        var eventSpace = (0, _height2.default)(this._firstRow) - labelHeight;

        this._needLimitMeasure = false;

        this.setState({
            needLimitMeasure: false,
            rowLimit: Math.max(Math.floor(eventSpace / eventHeight), 1)
        });
    };

    MonthView.prototype._dateClick = function _dateClick(date, e) {
        e.preventDefault();
        this.clearSelection();
        (0, _helpers.notify)(this.props.onNavigate, [_constants.navigate.DATE, date]);
    };

    MonthView.prototype._selectEvent = function _selectEvent() {
        //cancel any pending selections so only the event click goes through.
        this.clearSelection();

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        (0, _helpers.notify)(this.props.onSelectEvent, args);
    };

    MonthView.prototype._selectDates = function _selectDates(e) {
        var slots = this._pendingSelection.slice();

        this._pendingSelection = [];

        slots.sort(function (a, b) {
            return +a - +b;
        });

        (0, _helpers.notify)(this.props.onSelectSlot, {
            slots: slots,
            start: slots[0],
            end: slots[slots.length - 1]
        });
    };

    MonthView.prototype._showMore = function _showMore(segments, date, weekIdx, slot) {
        var cell = (0, _reactDom.findDOMNode)(this._bgRows[weekIdx]).children[slot - 1];

        var events = segments.filter(function (seg) {
            return isSegmentInSlot(seg, slot);
        }).map(function (seg) {
            return seg.event;
        });

        //cancel any pending selections so only the event click goes through.
        this.clearSelection();

        if (this.props.popup) {
            var position = (0, _position2.default)(cell, (0, _reactDom.findDOMNode)(this));

            this.setState({
                overlay: { date: date, events: events, position: position }
            });
        } else {
            (0, _helpers.notify)(this.props.onNavigate, [_constants.navigate.DATE, date]);
        }

        (0, _helpers.notify)(this.props.onShowMore, [events, date, slot]);
    };

    MonthView.prototype.clearSelection = function clearSelection() {
        clearTimeout(this._selectTimer);
        this._pendingSelection = [];
    };

    MonthView.prototype.render = function render() {
        var _this9 = this;

        var _props = this.props,
            date = _props.date,
            culture = _props.culture,
            weekdayFormat = _props.weekdayFormat,
            month = _dates3.default.visibleDays(date, culture),
            weeks = (0, _chunk2.default)(month, 7);

        var measure = this.state.needLimitMeasure;

        this._weekCount = weeks.length;

        return _react2.default.createElement(
            'div',
            { className: (0, _classnames2.default)('rbc-month-view', this.props.className) },
            _react2.default.createElement(
                'div',
                { className: 'rbc-row rbc-month-header' },
                this._headers(weeks[0], weekdayFormat, culture)
            ),
            weeks.map(function (week, idx) {
                return _this9.renderWeek(week, idx, measure && _this9._renderMeasureRows);
            }),
            this.props.popup && this._renderOverlay()
        );
    };

    return MonthView;
}(_react2.default.Component);

MonthView.navigate = function (date, action) {
    switch (action) {
        case _constants.navigate.PREVIOUS:
            return _dates3.default.add(date, -1, 'month');

        case _constants.navigate.NEXT:
            return _dates3.default.add(date, 1, 'month');

        default:
            return date;
    }
};

MonthView.range = function (date, _ref3) {
    var culture = _ref3.culture;

    var start = _dates3.default.firstVisibleDay(date, culture);
    var end = _dates3.default.lastVisibleDay(date, culture);
    return { start: start, end: end };
};

MonthView.propTypes = PROPERTY_TYPES;

exports.default = MonthView;