'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _utils = require('../util/utils');

var _sizes = require('../util/constant/sizes');

var sizes = _interopRequireWildcard(_sizes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    disabled: _propTypes2.default.bool,
    inputRef: _propTypes2.default.func,
    placeholder: _propTypes2.default.string,
    maxLength: _propTypes2.default.number,
    type: _propTypes2.default.oneOf(['text', 'password']),
    value: _propTypes2.default.any,
    onBlur: _propTypes2.default.func,
    onKeyPress: _propTypes2.default.func,
    onKeyDown: _propTypes2.default.func,
    onFocus: _propTypes2.default.func,
    onChange: _propTypes2.default.func,
    readOnly: _propTypes2.default.bool,
    fluid: _propTypes2.default.bool,
    autoFocus: _propTypes2.default.bool,
    size: sizes.anySize,
    testId: _propTypes2.default.any
};
var DEFAULT_PROPS = {
    inputRef: function inputRef(input) {},
    onFocus: function onFocus() {},
    type: 'text',
    maxLength: 254,
    autoFocus: false,
    readOnly: false,
    fluid: false,
    sizes: sizes.NORMAL
};

var Input = function (_React$Component) {
    _inherits(Input, _React$Component);

    function Input(props) {
        _classCallCheck(this, Input);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this._onFocus = function (e) {
            e.preventDefault();
            _this.props.onFocus(e);
        };

        _this.update = function (e) {
            var value = e.target.value;
            if (!_this.isControlled()) {
                _this.setState({
                    value: value
                });
            }
            if (_this.props.onChange && !_this.props.disabled) {
                _this.props.onChange(value);
            }
        };

        _this._attachInput = function (input) {
            _this._input = input;
            _this.props.inputRef(input);
        };

        if (!_this.isControlled()) {
            _this.state = {
                value: ''
            };
        }
        return _this;
    }

    Input.prototype.getValue = function getValue() {
        return this.state.value;
    };

    Input.prototype.focus = function focus() {
        //hoc is using this method
        this._searchInput.focus();
    };

    Input.prototype.isControlled = function isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    Input.prototype.render = function render() {
        var _this2 = this;

        var style = (0, _classnames2.default)(this.props.className, 'text-input', {
            disabled: this.props.disabled,
            'text-input--fluid': this.props.fluid,
            'text-input--sm': this.props.size === sizes.SMALL,
            'text-input--lg': this.props.size === sizes.LARGE
        });

        return _react2.default.createElement(
            'div',
            { className: style },
            _react2.default.createElement('input', {
                'data-test-id': this.props.testId,
                autoFocus: this.props.autoFocus,
                maxLength: this.props.maxLength,
                ref: this._attachInput,
                readOnly: this.props.readOnly,
                value: this.isControlled() ? this.props.value : this.state.value,
                placeholder: this.props.placeholder,
                onChange: function onChange(e) {
                    return _this2.update(e);
                },
                onKeyPress: this.props.onKeyPress,
                onKeyDown: this.props.onKeyDown,
                onBlur: this.props.onBlur,
                onFocus: this._onFocus,
                type: this.props.type,
                disabled: this.props.disabled
            })
        );
    };

    return Input;
}(_react2.default.Component);

Input.propTypes = PROPERTY_TYPES;
Input.defaultProps = DEFAULT_PROPS;

exports.default = Input;