'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Popup = require('../Popup/Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _MonthPickerPopup = require('./MonthPickerPopup');

var _MonthPickerPopup2 = _interopRequireDefault(_MonthPickerPopup);

var _WithIdAndTypeHOC = require('../util/hoc/WithIdAndTypeHOC');

var _dateUtils = require('../util/date-utils');

var dateUtils = _interopRequireWildcard(_dateUtils);

var _dateFormats = require('../util/date-formats');

var dateFormats = _interopRequireWildcard(_dateFormats);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    className: _propTypes2.default.string,
    baseClassName: _propTypes2.default.string,
    value: _propTypes2.default.string,
    format: _propTypes2.default.string,
    onChange: _propTypes2.default.func,
    minDate: _propTypes2.default.string,
    maxDate: _propTypes2.default.string,
    formatMonth: _propTypes2.default.func,
    target: _propTypes2.default.element
};

var DEFAULT_PROPS = {
    baseClassName: 'btn btn-link',
    formatMonth: dateFormats.formatShortMonthYear
};

var MonthPicker = function (_React$Component) {
    _inherits(MonthPicker, _React$Component);

    function MonthPicker(props) {
        _classCallCheck(this, MonthPicker);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            year: dateUtils.getYear(dateUtils.getToday()),
            value: props.value ? dateUtils.parse(props.value, props.format) : dateUtils.getFirstDayOfMonth(dateUtils.getToday()),
            open: false
        };

        _this.handleSelection = _this.handleSelection.bind(_this);
        _this.resetYear = _this.resetYear.bind(_this);
        return _this;
    }

    MonthPicker.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (nextProps.value) {
            this.setState({
                value: dateUtils.parse(nextProps.value, nextProps.format)
            });
        }
    };

    MonthPicker.prototype.formatValue = function formatValue() {
        return this.props.formatMonth(this.state.value);
    };

    MonthPicker.prototype.handleSelection = function handleSelection(selected) {
        var _this2 = this;

        this.setState({
            open: false
        }, function () {
            setTimeout(function () {
                if (!_this2.isControlled()) {
                    _this2.setState({
                        value: selected
                    });
                }

                if (_this2.props.onChange) {
                    _this2.props.onChange(_this2.doFormat(selected));
                }
            }, 200);
        });
    };

    MonthPicker.prototype.doFormat = function doFormat(date) {
        return dateUtils.format(date, this.props.format);
    };

    MonthPicker.prototype.doParse = function doParse(date) {
        return dateUtils.parse(date, this.props.format);
    };

    MonthPicker.prototype.resetYear = function resetYear() {
        this.setState({
            year: dateUtils.getYear(this.state.value)
        });
    };

    MonthPicker.prototype.isControlled = function isControlled() {
        if (this.props.value) {
            return true;
        } else {
            return false;
        }
    };

    MonthPicker.prototype.getStyles = function getStyles(date) {
        return (0, _classnames2.default)('month-picker__date-cell', {
            'month-picker__date-cell--selected': dateUtils.isMonthTheSame(date, this.state.value)
        });
    };

    MonthPicker.prototype.getTarget = function getTarget() {
        var target = _react2.default.Children.only(this.props.children);
        return _react2.default.cloneElement(target, {
            id: this.props.id,
            value: this.formatValue()
        });
    };

    MonthPicker.prototype.render = function render() {
        var _this3 = this;

        return _react2.default.createElement(
            _Popup2.default,
            {
                popupClassName: 'popover-colored',
                attachment: 'bottom center',
                on: 'click',
                onPopupStateChange: function onPopupStateChange(open) {
                    return _this3.setState({ open: open });
                },
                open: this.state.open,
                onTransitionClosedToOpen: function onTransitionClosedToOpen() {
                    _this3.resetYear();
                }
            },
            this.getTarget(),
            _react2.default.createElement(_MonthPickerPopup2.default, {
                year: this.state.year,
                onSelected: this.handleSelection,
                onYearChange: function onYearChange(year) {
                    return _this3.setState({ year: year });
                },
                minDate: this.props.minDate ? this.doParse(this.props.minDate) : undefined,
                maxDate: this.props.maxDate ? this.doParse(this.props.maxDate) : undefined,
                getStyles: function getStyles(date) {
                    return _this3.getStyles(date);
                }
            })
        );
    };

    return MonthPicker;
}(_react2.default.Component);

MonthPicker.propTypes = PROPERTY_TYPES;
MonthPicker.defaultProps = DEFAULT_PROPS;

exports.default = (0, _WithIdAndTypeHOC.withIdAndTypeContext)(MonthPicker, 'MonthPicker');