'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _chunk = require('lodash/chunk');

var _chunk2 = _interopRequireDefault(_chunk);

var _dateUtils = require('../../util/date-utils');

var dateUtils = _interopRequireWildcard(_dateUtils);

var _dateFormats = require('../../util/date-formats');

var dateFormats = _interopRequireWildcard(_dateFormats);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MONTHS_IN_ROW = 4;

var PROPERTY_TYPES = {
    getStyles: _propTypes2.default.func,
    onSelected: _propTypes2.default.func,
    onMouseEnter: _propTypes2.default.func,
    onMouseLeave: _propTypes2.default.func,
    formatDataDate: _propTypes2.default.func,
    parseDataDate: _propTypes2.default.func,
    formatMonth: _propTypes2.default.func,
    year: _propTypes2.default.number.isRequired,
    componentWillUnmount: _propTypes2.default.func,
    minDate: _propTypes2.default.object,
    maxDate: _propTypes2.default.object
};

var DEFAULT_PROPS = {
    formatDataDate: _dateFormats.formatDDMMYYYY,
    parseDataDate: _dateFormats.parseDDMMYYYY,
    formatMonth: dateFormats.formatShortMonth
};

var MonthPickerCalendar = function (_React$Component) {
    _inherits(MonthPickerCalendar, _React$Component);

    function MonthPickerCalendar(props) {
        _classCallCheck(this, MonthPickerCalendar);

        return _possibleConstructorReturn(this, _React$Component.call(this, props));
    }

    MonthPickerCalendar.prototype.componentWillUnmount = function componentWillUnmount() {
        if (this.props.componentWillUnmount) {
            this.props.componentWillUnmount();
        }
    };

    MonthPickerCalendar.prototype.retrieveDate = function retrieveDate(e) {
        var selected = e.currentTarget.getAttribute('data-date');
        return this.parseDataDate(selected);
    };

    MonthPickerCalendar.prototype.handleClick = function handleClick(e) {
        var date = this.retrieveDate(e);
        if (this.isDisabled(date)) {
            return;
        }

        if (this.props.onSelected) {
            this.props.onSelected(date);
        }
    };

    MonthPickerCalendar.prototype.handleMouseEnter = function handleMouseEnter(e) {
        if (this.props.onMouseEnter) {
            this.props.onMouseEnter(this.retrieveDate(e));
        }
    };

    MonthPickerCalendar.prototype.handleMouseLeave = function handleMouseLeave(e) {
        if (this.props.onMouseLeave) {
            this.props.onMouseLeave();
        }
    };

    MonthPickerCalendar.prototype.getDateStyles = function getDateStyles(date) {
        if (this.props.getStyles) {
            return this.props.getStyles(date);
        }
    };

    MonthPickerCalendar.prototype.parseDataDate = function parseDataDate(date) {
        return this.props.parseDataDate(date);
    };

    MonthPickerCalendar.prototype.formatDataDate = function formatDataDate(date) {
        return this.props.formatDataDate(date);
    };

    MonthPickerCalendar.prototype.formatDate = function formatDate(date) {
        return this.props.formatDate(date);
    };

    MonthPickerCalendar.prototype.renderMonths = function renderMonths() {
        var _this2 = this;

        var months = (0, _chunk2.default)(dateUtils.getFirstDaysOfAllMonths(this.props.year), MONTHS_IN_ROW);
        return months.map(function (month, index) {
            return _react2.default.createElement(
                'tr',
                { key: 'month-' + index },
                _this2.renderRow(month)
            );
        });
    };

    MonthPickerCalendar.prototype.isDisabled = function isDisabled(month) {
        if (this.props.minDate && dateUtils.isMonthBefore(month, this.props.minDate)) {
            return true;
        }

        if (this.props.maxDate && dateUtils.isMonthAfter(month, this.props.maxDate)) {
            return true;
        }
    };

    MonthPickerCalendar.prototype.renderRow = function renderRow(month) {
        var _this3 = this;

        return month.map(function (month) {
            var className = (0, _classnames2.default)('month-picker-calendar__month-cell', _this3.getDateStyles(month), {
                'month-picker-calendar__month-cell--today': dateUtils.isCurrentMonth(month)
            });

            var formattedMonth = _this3.formatDataDate(month);

            var props = {
                key: formattedMonth,
                onClick: function onClick(e) {
                    return _this3.handleClick(e);
                },
                onMouseEnter: function onMouseEnter(e) {
                    return _this3.handleMouseEnter(e);
                },
                onMouseLeave: function onMouseLeave(e) {
                    return _this3.handleMouseLeave(e);
                },
                className: className,
                'data-date': formattedMonth
            };

            if (_this3.isDisabled(month)) {
                props['data-disabled'] = true;
            }

            return _react2.default.createElement(
                'td',
                props,
                _react2.default.createElement(
                    'div',
                    null,
                    _this3.props.formatMonth(month)
                )
            );
        });
    };

    MonthPickerCalendar.prototype.render = function render() {
        return _react2.default.createElement(
            'div',
            { className: 'month-picker-calendar' },
            _react2.default.createElement(
                'table',
                { className: 'month-picker-calendar__grid' },
                _react2.default.createElement(
                    'tbody',
                    null,
                    this.renderMonths()
                )
            )
        );
    };

    return MonthPickerCalendar;
}(_react2.default.Component);

MonthPickerCalendar.propTypes = PROPERTY_TYPES;
MonthPickerCalendar.defaultProps = DEFAULT_PROPS;

exports.default = MonthPickerCalendar;