'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _range = require('lodash/range');

var _range2 = _interopRequireDefault(_range);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Select = require('../../Select/Select');

var _Select2 = _interopRequireDefault(_Select);

var _MonthPickerCalendar = require('../MonthPickerCalendar/MonthPickerCalendar');

var _MonthPickerCalendar2 = _interopRequireDefault(_MonthPickerCalendar);

var _Button = require('../../Button/Button');

var _Button2 = _interopRequireDefault(_Button);

var _dateUtils = require('../../util/date-utils');

var dateUtils = _interopRequireWildcard(_dateUtils);

var _utils = require('../../util/utils');

var utils = _interopRequireWildcard(_utils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    value: _propTypes2.default.object,
    year: _propTypes2.default.number,
    onSelected: _propTypes2.default.func.isRequired,
    onYearChange: _propTypes2.default.func,
    minDate: _propTypes2.default.object,
    maxDate: _propTypes2.default.object,
    getStyles: _propTypes2.default.func
};

var DEFAULT_PROPS = {
    value: dateUtils.getToday(),
    year: dateUtils.getYear(dateUtils.getToday()),
    minDate: dateUtils.getFirstDayOfYear(dateUtils.addYears(dateUtils.getToday(), -3)),
    maxDate: dateUtils.getLastDayOfYear(dateUtils.addYears(dateUtils.getToday(), 2))
};

var DatePickerPopup = function (_React$Component) {
    _inherits(DatePickerPopup, _React$Component);

    function DatePickerPopup(props) {
        _classCallCheck(this, DatePickerPopup);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            year: props.year,
            value: props.value,
            years: (0, _range2.default)(dateUtils.getYear(props.minDate), dateUtils.getYear(props.maxDate) + 1).map(function (item) {
                return { id: item, displayString: '' + item };
            })
        };
        return _this;
    }

    DatePickerPopup.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (nextProps.year) {
            this.setState({
                year: nextProps.year
            });
        }
    };

    DatePickerPopup.prototype.left = function left() {
        if (this.state.year <= dateUtils.getYear(this.props.minDate)) {
            return;
        }
        this.updateMonth(this.state.year - 1);
    };

    DatePickerPopup.prototype.right = function right() {
        if (this.state.year >= dateUtils.getYear(this.props.maxDate)) {
            return;
        }
        this.updateMonth(this.state.year + 1);
    };

    DatePickerPopup.prototype.onYearChange = function onYearChange(selectedYear) {
        this.updateMonth(selectedYear);
    };

    DatePickerPopup.prototype.updateMonth = function updateMonth(year) {
        if (this.isYearControlled()) {
            this.props.onYearChange(year);
        } else {
            this.setState({
                year: year
            });
        }
    };

    DatePickerPopup.prototype.isYearControlled = function isYearControlled() {
        if (utils.isDefined(this.props.year)) {
            return true;
        } else {
            return false;
        }
    };

    DatePickerPopup.prototype.render = function render() {
        var _this2 = this;

        return _react2.default.createElement(
            'div',
            { tabIndex: '-1' },
            _react2.default.createElement(
                'div',
                { className: 'month-picker-popup__header' },
                _react2.default.createElement('i', {
                    onClick: function onClick() {
                        return _this2.left();
                    },
                    className: 'im month-picker-popup__arrow month-picker-popup__arrow--left'
                }),
                _react2.default.createElement(_Select2.default, {
                    className: 'month-picker-popup__years select--sm',
                    value: this.state.year,
                    onChange: function onChange(year) {
                        _this2.onYearChange(year);
                    },
                    options: this.state.years
                }),
                _react2.default.createElement('i', {
                    onClick: function onClick() {
                        return _this2.right();
                    },
                    className: 'im month-picker-popup__arrow month-picker-popup__arrow--right'
                })
            ),
            _react2.default.createElement(_MonthPickerCalendar2.default, {
                onSelected: this.props.onSelected,
                year: this.state.year,
                minDate: this.props.minDate,
                maxDate: this.props.maxDate,
                getStyles: this.props.getStyles
            }),
            _react2.default.createElement(
                _Button2.default,
                {
                    onClick: function onClick() {
                        return _this2.props.onSelected(dateUtils.getFirstDayOfMonth(dateUtils.getToday()));
                    },
                    className: 'month-picker-popup__today-link btn-link btn--sm'
                },
                'Current month'
            )
        );
    };

    return DatePickerPopup;
}(_react2.default.Component);

DatePickerPopup.propTypes = PROPERTY_TYPES;
DatePickerPopup.defaultProps = DEFAULT_PROPS;

exports.default = DatePickerPopup;