'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.PROPERTY_TYPES = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Popup = require('../Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _utils = require('../util/utils');

var _MultiSelectPopup = require('./MultiSelectPopup');

var _MultiSelectPopup2 = _interopRequireDefault(_MultiSelectPopup);

var _WithIdAndTypeHOC = require('../util/hoc/WithIdAndTypeHOC');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = exports.PROPERTY_TYPES = {
    disabled: _propTypes2.default.bool,
    placeholder: _propTypes2.default.string,
    searchPlaceholder: _propTypes2.default.string,
    options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.number.isRequired,
        displayString: _propTypes2.default.string.isRequired
    })),
    className: _propTypes2.default.string,
    onChange: _propTypes2.default.func,
    handleTooltipStateChange: _propTypes2.default.func,
    getText: _propTypes2.default.func,

    showSearch: _propTypes2.default.bool,
    showClear: _propTypes2.default.bool,
    popupHeader: _propTypes2.default.node,

    selectedOnTop: _propTypes2.default.bool
};
var DEFAULT_PROPS = {
    placeholder: 'Select...',
    searchPlaceholder: 'Search ...',
    getText: function getText(selectedOptions) {
        return selectedOptions.length + ' item(s) selected';
    },
    selectedOnTop: true,
    disabled: false
};

var MultiSelect = function (_React$Component) {
    _inherits(MultiSelect, _React$Component);

    function MultiSelect(props) {
        _classCallCheck(this, MultiSelect);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            selected: _this.sort((0, _utils.findIdentifiables)(_this.props.options, props.value)),
            popupOpen: false,
            tooltipOpen: false
        };
        return _this;
    }

    MultiSelect.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        var isValueChanged = nextProps.value !== this.props.value;
        if (isValueChanged) {
            this.setState({ selected: this.sort((0, _utils.findIdentifiables)(nextProps.options, nextProps.value)) });
        }
    };

    MultiSelect.prototype.select = function select(selected) {
        if (this.props.onChange) {
            this.props.onChange(selected.map(function (option) {
                return option.id;
            }), selected);
        }
        this.setState({
            selected: this.sort(selected)
        });
    };

    MultiSelect.prototype.sort = function sort(selected) {
        return selected.sort(function (left, right) {
            return (0, _utils.compare)(left.displayString, right.displayString);
        });
    };

    MultiSelect.prototype.handlePopupStateChange = function handlePopupStateChange(open) {
        if (this.props.disabled) {
            return;
        }

        if (this.props.handlePopupStateChange) {
            this.props.handlePopupStateChange(open);
        }

        var newState = {
            popupOpen: open,
            tooltipOpen: false
        };

        this.setState(newState);
        if (open) {
            this.handlePopupOpening();
        }
    };

    MultiSelect.prototype.handleTooltipStateChange = function handleTooltipStateChange(open) {
        if (this.props.handleTooltipStateChange) {
            this.props.handleTooltipStateChange(open);
        }

        if (!this.state.popupOpen && this.state.selected.length > 0) {
            this.setState({ tooltipOpen: open });
        }
    };

    MultiSelect.prototype.isControlled = function isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    MultiSelect.prototype.handlePopupOpening = function handlePopupOpening() {
        var popupContent = _reactDom2.default.findDOMNode(this.refs.popup.getContent());
        var input = _reactDom2.default.findDOMNode(this.refs.input);
        popupContent.style.width = input.offsetWidth + 'px';
    };

    MultiSelect.prototype.getText = function getText() {
        if (this.state.selected.length === 0) {
            return this.props.placeholder;
        } else {
            return this.props.getText(this.state.selected);
        }
    };

    MultiSelect.prototype.render = function render() {
        var _this2 = this;

        var className = (0, _classnames2.default)(this.props.className, 'select', {
            placeholder: this.state.selected.length === 0,
            disabled: this.props.disabled
        });

        return _react2.default.createElement(
            _Popup2.default,
            {
                ref: 'popup',
                attachment: 'bottom left',
                on: 'click',
                popupClassName: 'select__popover',
                animationBaseName: 'select__popover--animation-slide-y',
                open: this.state.popupOpen,
                onPopupStateChange: function onPopupStateChange(open) {
                    return _this2.handlePopupStateChange(open);
                }
            },
            _react2.default.createElement(
                _Popup2.default,
                {
                    attachment: 'bottom left',
                    on: 'hover',
                    popupClassName: 'tooltip',
                    open: this.state.tooltipOpen,
                    onPopupStateChange: function onPopupStateChange(open) {
                        return _this2.handleTooltipStateChange(open);
                    }
                },
                _react2.default.createElement(
                    'div',
                    { id: this.props.id, ref: 'input', tabIndex: '0', className: className },
                    _react2.default.createElement(
                        'div',
                        null,
                        this.getText()
                    ),
                    _react2.default.createElement('i', { className: 'im icon-arrow-down' })
                ),
                _react2.default.createElement(
                    'div',
                    null,
                    this.state.selected.map(function (option) {
                        return _react2.default.createElement(
                            'div',
                            { key: 'tooltip-' + option.id },
                            option.displayString
                        );
                    })
                )
            ),
            _react2.default.createElement(_MultiSelectPopup2.default, {
                selectedOnTop: this.props.selectedOnTop,
                showSearch: this.props.showSearch,
                showClear: this.props.showClear,
                popupHeader: this.props.popupHeader,
                placeholder: this.props.searchPlaceholder,
                options: this.props.options,
                value: this.state.selected.map(function (selected) {
                    return selected.id;
                }),
                onChange: function onChange(selectedIds, selected) {
                    return _this2.select(selected);
                },
                compare: _utils.compare,
                renderOptions: this.props.renderOptions
            })
        );
    };

    return MultiSelect;
}(_react2.default.Component);

MultiSelect.propTypes = PROPERTY_TYPES;
MultiSelect.defaultProps = DEFAULT_PROPS;

exports.default = (0, _WithIdAndTypeHOC.withIdAndTypeContext)(MultiSelect, 'MultiSelect');