'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _trim = require('lodash/trim');

var _trim2 = _interopRequireDefault(_trim);

var _fuse = require('fuse.js');

var _fuse2 = _interopRequireDefault(_fuse);

var _Link = require('../Link/Link');

var _Link2 = _interopRequireDefault(_Link);

var _utils = require('../util/utils');

var _IconInput = require('../InputIcon/IconInput');

var _IconInput2 = _interopRequireDefault(_IconInput);

var _CheckBoxList = require('../Select/CheckBoxList');

var _CheckBoxList2 = _interopRequireDefault(_CheckBoxList);

var _KeyBoardNav = require('../Select/KeyBoardNav');

var _KeyBoardNav2 = _interopRequireDefault(_KeyBoardNav);

var _fuseSearchConfig = require('../Select/fuseSearchConfig');

var _fuseSearchConfig2 = _interopRequireDefault(_fuseSearchConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    placeholder: _propTypes2.default.string,
    options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.number.isRequired,
        displayString: _propTypes2.default.string.isRequired
    })),
    onChange: _propTypes2.default.func,
    value: _propTypes2.default.arrayOf(_propTypes2.default.number.isRequired),
    compare: _propTypes2.default.func,
    showSearch: _propTypes2.default.bool,
    showClear: _propTypes2.default.bool,
    selectedOnTop: _propTypes2.default.bool,
    popupHeader: _propTypes2.default.element
};
var DEFAULT_PROPS = {
    placeHolder: 'Search ...',
    compare: _utils.compare,
    showSearch: true,
    showClear: true,
    popupHeader: null,
    selectedOnTop: true
};

var MultiSelectPopup = function (_React$Component) {
    _inherits(MultiSelectPopup, _React$Component);

    function MultiSelectPopup(props) {
        _classCallCheck(this, MultiSelectPopup);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this._applySearch = function (value) {
            if (_this.state.query === value) {
                return;
            }
            _this.resetNav();
            var filtered = (0, _trim2.default)(value).length === 0 ? _this.props.options : _this.fuse.search(value);

            if (_this.props.selectedOnTop) {
                filtered.sort(_this._sortSelectedOnTop);
            }

            _this.setState({
                filtered: filtered,
                query: value
            });
        };

        _this.resetSearch = function () {
            var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.props.options;

            _this.resetNav();
            _this.setState({
                filtered: _this.props.selectedOnTop ? options.sort(_this._sortSelectedOnTop) : options,
                query: ''
            });
        };

        _this.renderOptions = function () {
            return _react2.default.createElement(
                'div',
                { className: 'select__popup' },
                _this.props.popupHeader ? _this.props.popupHeader : null,
                _this.props.showSearch ? _react2.default.createElement(
                    'div',
                    { className: 'select__search-container' },
                    _react2.default.createElement(_IconInput2.default, {
                        inputRef: function inputRef(input) {
                            return _this._searchInput = input;
                        },
                        onKeyDown: function onKeyDown(e) {
                            return _this.navigate(e);
                        },
                        ref: 'searchInput',
                        fluid: true,
                        placeholder: _this.props.placeholder,
                        size: 'sm',
                        onChange: _this._applySearch,
                        className: 'select__search',
                        iconClassName: 'im icon-search'
                    })
                ) : null,
                _this.props.showClear ? _react2.default.createElement(
                    'div',
                    null,
                    _react2.default.createElement(
                        'div',
                        { className: 'select__clear-btn' },
                        _react2.default.createElement(
                            _Link2.default,
                            {
                                className: 'select__clear-btn',
                                onClick: function onClick() {
                                    _this._selectOptions([]);
                                }
                            },
                            'Clear all selected'
                        )
                    )
                ) : null,
                _react2.default.createElement(
                    'div',
                    { className: 'select__options' },
                    _react2.default.createElement(_CheckBoxList2.default, {
                        ref: function ref(checkBoxList) {
                            return _this._checkBoxList = checkBoxList;
                        },
                        focus: _this.state.tempSelection,
                        options: _this.state.filtered,
                        onChange: function onChange(selected) {
                            return _this._selectOptions(selected);
                        },
                        value: _this.state.selected.map(function (selected) {
                            return selected.id;
                        })
                    })
                )
            );
        };

        _this._sortSelectedOnTop = _this._sortSelectedOnTop.bind(_this);
        _this.getOptions = _this.getOptions.bind(_this);
        _this.state = {
            selected: (0, _utils.findIdentifiables)(_this.props.options, props.value)
        };

        _this.fuse = new _fuse2.default(props.options, _fuseSearchConfig2.default);

        _this.state.filtered = _this.getAllSorted();
        return _this;
    }

    MultiSelectPopup.prototype.componentDidMount = function componentDidMount() {
        var _this2 = this;

        if (this.refs.searchInput && this.refs.searchInput.focus) {
            setTimeout(function () {
                return _this2.refs.searchInput.focus();
            }, 0);
        }
    };

    MultiSelectPopup.prototype.getAllSorted = function getAllSorted() {
        var result = this.props.options.slice(0);
        if (this.props.selectedOnTop) {
            result.sort(this._sortSelectedOnTop);
        }
        return result;
    };

    MultiSelectPopup.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        this.fuse.setCollection(nextProps.options);

        var isValueChanged = nextProps.value !== this.props.value;
        if (isValueChanged) {
            if (this.props.options !== nextProps.options) {
                this.resetSearch(nextProps.options);
            }
            this.setState({ selected: (0, _utils.findIdentifiables)(nextProps.options, nextProps.value) });
        }
    };

    MultiSelectPopup.prototype.isSelected = function isSelected(option) {
        return this.findOptionInList(this.state.selected, option);
    };

    MultiSelectPopup.prototype.findOptionInList = function findOptionInList(list, option) {
        return (0, _find2.default)(list, function (item) {
            return item.id === option.id;
        });
    };

    MultiSelectPopup.prototype.mergeSelectedOptions = function mergeSelectedOptions(selected) {
        var _this3 = this;

        return this.state.selected.filter(function (item) {
            var found = _this3.findOptionInList(_this3.state.filtered, item);
            return !found;
        }).concat(selected);
    };

    MultiSelectPopup.prototype.select = function select(option) {
        var beforeLen = this.state.selected.length;

        var selected = this.state.selected.filter(function (item) {
            return item.id !== option.id;
        });

        if (beforeLen === selected.length) {
            selected.push(option);
        }

        this._selectOptions(selected);
    };

    MultiSelectPopup.prototype._selectOptions = function _selectOptions(selected) {
        var selectedMerged = this.mergeSelectedOptions(selected);

        if (!this._isControlled()) {
            this.setState({
                selected: selectedMerged
            });
        }

        if (this.getInput()) {
            this.getInput().focus();
        }

        if (this.props.onChange) {
            this.props.onChange(selectedMerged.map(function (option) {
                return option.id;
            }), selectedMerged);
        }
    };

    MultiSelectPopup.prototype._sortSelectedOnTop = function _sortSelectedOnTop(option1, option2) {
        var isOption1Selected = this.isSelected(option1);
        var isOption2Selected = this.isSelected(option2);

        if (isOption1Selected && !isOption2Selected) {
            return -1;
        } else if (isOption2Selected && !isOption1Selected) {
            return 1;
        } else {
            return this.props.compare(option1.displayString, option2.displayString);
        }
    };

    MultiSelectPopup.prototype._isControlled = function _isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    MultiSelectPopup.prototype.getOptions = function getOptions() {
        return this.state.filtered;
    };

    MultiSelectPopup.prototype.getSelectedOptionEl = function getSelectedOptionEl() {
        return this._checkBoxList.getFocusedEl();
    };

    MultiSelectPopup.prototype.getInput = function getInput() {
        return this._searchInput;
    };

    MultiSelectPopup.prototype.render = function render() {
        return this.props.renderOptions ? this.props.renderOptions() : this.renderOptions();
    };

    return MultiSelectPopup;
}(_react2.default.Component);

MultiSelectPopup.propTypes = PROPERTY_TYPES;
MultiSelectPopup.defaultProps = DEFAULT_PROPS;

exports.default = (0, _KeyBoardNav2.default)(true)(MultiSelectPopup);