'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.attachmentPositions = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _noop2 = require('lodash/noop');

var _noop3 = _interopRequireDefault(_noop2);

var _CSSTransitionGroup = require('react-transition-group/CSSTransitionGroup');

var _CSSTransitionGroup2 = _interopRequireDefault(_CSSTransitionGroup);

var _TetherComponent = require('../react-tether/TetherComponent.js');

var _TetherComponent2 = _interopRequireDefault(_TetherComponent);

var _keyCodes = require('../util/constant/key-codes');

var _OnClickOutsideHOC = require('../util/hoc/OnClickOutsideHOC');

var _OnKeyDownHOC = require('../util/hoc/OnKeyDownHOC');

var _utils = require('../util/utils');

var _PopupElement = require('./PopupElement');

var _PopupElement2 = _interopRequireDefault(_PopupElement);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var attachmentPositions = exports.attachmentPositions = {
    'top left': { attachment: 'bottom left', targetAttachment: 'top left' },
    'top center': { attachment: 'bottom center', targetAttachment: 'top center' },
    'top right': { attachment: 'bottom right', targetAttachment: 'top right' },
    'middle left': { attachment: 'middle right', targetAttachment: 'middle left' },
    'middle right': { attachment: 'middle left', targetAttachment: 'middle right' },
    'bottom left': { attachment: 'top left', targetAttachment: 'bottom left' },
    'bottom center': { attachment: 'top center', targetAttachment: 'bottom center' },
    'bottom right': { attachment: 'top right', targetAttachment: 'bottom right' },
    'middle center': { attachment: 'middle center', targetAttachment: 'middle center' }
};

var POPUP_STATE = {
    OPEN: 'open',
    CLOSING: 'closing',
    CLOSED: 'closed'
};

var PROPERTY_TYPES = {
    attachment: _propTypes2.default.oneOf(Object.keys(attachmentPositions)).isRequired,
    on: _propTypes2.default.oneOf(['hover', 'click', 'focus']),
    popupClassName: _propTypes2.default.string,
    targetClassName: _propTypes2.default.string,
    overlayClassName: _propTypes2.default.string,
    changeAttachmentDynamically: _propTypes2.default.bool,
    modal: _propTypes2.default.bool,
    animationBaseName: _propTypes2.default.string,
    onOpen: _propTypes2.default.func,
    onClose: _propTypes2.default.func,
    onTransitionClosedToOpen: _propTypes2.default.func,
    onPopupStateChange: _propTypes2.default.func,
    open: _propTypes2.default.bool,
    openByDefault: _propTypes2.default.bool,
    onContentDidMount: _propTypes2.default.func,
    closeOnClickOutside: _propTypes2.default.bool,
    animate: _propTypes2.default.bool,
    popupWidth: _propTypes2.default.number
};
var DEFAULT_PROPS = {
    on: 'hover',
    popupClassName: 'popover',
    targetClassName: '',
    changeAttachmentDynamically: false,
    modal: false,
    animationBaseName: 'popup--animation-scale',
    onContentDidMount: _noop3.default,
    onClose: _noop3.default,
    animate: true
};

var Popup = function (_React$Component) {
    _inherits(Popup, _React$Component);

    function Popup(props) {
        _classCallCheck(this, Popup);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.position = function () {
            if (!_this.tether) {
                return;
            }

            var tetherInstance = _this.tether.getTetherInstance();
            tetherInstance && tetherInstance.position();
        };

        _this.handleRepositioned = function () {
            for (var i = 0; i < _this.popupsToRepopsition.length; i++) {
                _this.popupsToRepopsition[i]();
            }
        };

        _this.state = {
            popupState: (0, _utils.isDefined)(props.open) ? props.open ? POPUP_STATE.OPEN : POPUP_STATE.CLOSED : props.openByDefault ? POPUP_STATE.OPEN : POPUP_STATE.CLOSED
        };
        _this.overlayElement = null;
        _this.openTimeout = null;
        _this.blurTimeout = null;
        _this.additionalChecks = [];
        _this.popupsToRepopsition = [];

        _this.doCheck = _this.doCheck.bind(_this);
        _this.isContainedByPopup = _this.isContainedByPopup.bind(_this);
        _this.close = _this.close.bind(_this);
        return _this;
    }

    Popup.prototype.componentDidMount = function componentDidMount() {
        if (this.context.addCheck) {
            this.context.addCheck(this.doCheck);
        }

        if (this.context.addPopupsToReposition) {
            this.context.addPopupsToReposition(this.position);
        }

        this.applyOverlay(this.state.popupState);
    };

    Popup.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if ((0, _utils.isDefined)(nextProps.open)) {
            var isOpen = nextProps.open;

            if (isOpen && POPUP_STATE.OPEN !== this.state.popupState) {
                this.updatePopupState(POPUP_STATE.OPEN);
            } else if (!isOpen && POPUP_STATE.OPEN === this.state.popupState) {
                this.updatePopupState(POPUP_STATE.CLOSING);
            }
        }
    };

    Popup.prototype.componentWillUnmount = function componentWillUnmount() {
        if (this.state.popupState !== POPUP_STATE.CLOSED) {
            this.closeCompletely();
        }

        if (this.openTimeout) {
            clearTimeout(this.openTimeout);
        }
        if (this.blurTimeout) {
            window.clearTimeout(this.blurTimeout);
            this.blurTimeout = null;
        }
    };

    Popup.prototype.updatePopupState = function updatePopupState(popupState) {
        this.setState({ popupState: popupState });
        this.applyOverlay(popupState);
    };

    Popup.prototype.getChildContext = function getChildContext() {
        var _this2 = this;

        return {
            addCheck: function addCheck(func) {
                _this2.additionalChecks.push(func);
            },
            addPopupsToReposition: function addPopupsToReposition(func) {
                _this2.popupsToRepopsition.push(func);
            }
        };
    };

    Popup.prototype.applyOverlay = function applyOverlay(popupState) {
        if (this.isModal()) {
            var overlayElement = this.overlayElement;
            var modalPopUpOverlayClassNames = (0, _classnames2.default)({
                'popup-overlay-opening': this.isModal() && popupState === POPUP_STATE.OPEN,
                'popup-overlay-closing': this.isModal() && popupState === POPUP_STATE.CLOSING,
                'popup-overlay': this.isModal(),
                'popup-overlay-transparent': this.props.closeOnClickOutside
            }, this.props.overlayClassName);

            switch (popupState) {
                case POPUP_STATE.OPEN:
                    if (overlayElement == null) {
                        overlayElement = document.createElement('div');
                    }
                    overlayElement.className = modalPopUpOverlayClassNames;
                    this.overlayElement = overlayElement;
                    document.body.appendChild(overlayElement);
                    break;
                case POPUP_STATE.CLOSING:
                    if (overlayElement != null) {
                        overlayElement.className = modalPopUpOverlayClassNames;
                    }
                    break;
                case POPUP_STATE.CLOSED:
                    if (overlayElement != null) {
                        document.body.removeChild(overlayElement);
                        this.overlayElement = null;
                    }
                    break;
                default:
                    break;
            }
        }
    };

    Popup.prototype.changeState = function changeState(popupState) {
        if (POPUP_STATE.CLOSED !== popupState && this.props.onPopupStateChange) {
            this.props.onPopupStateChange(POPUP_STATE.OPEN === popupState);
        }

        if (POPUP_STATE.CLOSED === popupState || !this.isPopupStateControlled()) {
            this.updatePopupState(popupState);
        }

        if (POPUP_STATE.CLOSING === popupState && this.isPopupStateControlled()) {
            this.props.onClose();
        }
    };

    Popup.prototype.isPopupStateControlled = function isPopupStateControlled() {
        return (0, _utils.isDefined)(this.props.open);
    };

    Popup.prototype.scheduleOpen = function scheduleOpen() {
        var _this3 = this;

        if (this.isOpen()) {
            return;
        }

        if (!this.isClosing()) {
            if (this.props.onTransitionClosedToOpen) {
                this.props.onTransitionClosedToOpen();
            }
        }

        this.openTimeout = setTimeout(function () {
            _this3.changeState(POPUP_STATE.OPEN);
            _this3.openTimeout = null;
            if (_this3.props.onOpen) {
                _this3.props.onOpen();
            }
        }, 50);
    };

    Popup.prototype.handleClickOutside = function handleClickOutside(e) {
        if (this.isModal() && !this.props.closeOnClickOutside) {
            return;
        }

        if (!this.isOpen()) {
            return;
        }

        if (this.isContainedByPopup(e.target)) {
            return;
        }

        if (this.applyAdditionalChecks(e.target)) {
            return;
        }

        this.close();
    };

    Popup.prototype.doCheck = function doCheck(element) {
        if (this.isContainedByPopup(element)) {
            return true;
        }

        return this.applyAdditionalChecks(element);
    };

    Popup.prototype.handleKeyDown = function handleKeyDown(keyCode) {
        if (keyCode === _keyCodes.ESCAPE && this.isOpen()) {
            this.close();
        }
    };

    Popup.prototype.close = function close() {
        if (this.openTimeout) {
            clearTimeout(this.openTimeout);
            this.openTimeout = null;
            this.changeState(POPUP_STATE.CLOSED);
        } else {
            this.changeState(POPUP_STATE.CLOSING);
        }
    };

    Popup.prototype.getContent = function getContent() {
        return this._content;
    };

    Popup.prototype.closeCompletely = function closeCompletely() {
        if (this.state.popupState !== POPUP_STATE.CLOSED) {
            this.changeState(POPUP_STATE.CLOSED);
            this.props.onClose();
        }
    };

    Popup.prototype.isOpen = function isOpen() {
        return this.state.popupState === POPUP_STATE.OPEN;
    };

    Popup.prototype.isClosing = function isClosing() {
        return this.state.popupState === POPUP_STATE.CLOSING;
    };

    Popup.prototype.isClosed = function isClosed() {
        return this.state.popupState === POPUP_STATE.CLOSED;
    };

    Popup.prototype.isModal = function isModal() {
        return this.props.modal === true;
    };

    Popup.prototype.toggle = function toggle() {
        if (this.isOpen()) {
            this.close();
        } else {
            this.scheduleOpen();
        }
    };

    Popup.prototype.isComponentChild = function isComponentChild(element) {
        return this.isContainedByComponent(element) || this.isContainedByPopup(element);
    };

    Popup.prototype.isContainedByPopup = function isContainedByPopup(element) {
        var domNode = _reactDom2.default.findDOMNode(this.refs.popup);
        if (domNode && domNode.parentNode.contains(element)) {
            return true;
        }
        return false;
    };

    Popup.prototype.applyAdditionalChecks = function applyAdditionalChecks(element) {
        for (var i = 0; i < this.additionalChecks.length; i++) {
            if (this.additionalChecks[i](element)) {
                return true;
            }
        }
        return false;
    };

    Popup.prototype.isContainedByComponent = function isContainedByComponent(element) {
        var currentNode = _reactDom2.default.findDOMNode(this);
        if (currentNode && currentNode.contains(element)) {
            return true;
        }
        return false;
    };

    Popup.prototype.getHandlers = function getHandlers() {
        var _this4 = this;

        if (this.props.on === 'focus') {
            return {
                onClick: function onClick() {
                    _this4.scheduleOpen();
                },
                onFocus: function onFocus() {
                    _this4.scheduleOpen();
                },
                onBlur: function onBlur() {
                    _this4.blurTimeout = setTimeout(function () {
                        var focused = document.activeElement;

                        if (_this4.isComponentChild(focused)) {
                            return;
                        }

                        _this4.close();
                        clearTimeout(_this4.blurTimeout);
                        _this4.blurTimeout = null;
                    }, 50);
                }
            };
        } else if (this.props.on === 'hover') {
            return {
                onMouseEnter: function onMouseEnter() {
                    return _this4.scheduleOpen();
                },
                onMouseLeave: function onMouseLeave() {
                    return _this4.close();
                }
            };
        } else {
            return {
                onClick: function onClick() {
                    return _this4.toggle();
                }
            };
        }
    };

    Popup.prototype.renderTarget = function renderTarget() {
        var _this5 = this;

        var target = _react2.default.Children.toArray(this.props.children)[0];
        var handlers = this.getHandlers();

        var targetClassNames = (0, _classnames2.default)(target.props.className, this.props.targetClassName, {
            open: this.isOpen()
        });

        target = _react2.default.cloneElement(target, _extends({}, handlers, {
            onClick: function onClick() {
                if (_this5.props.onClick) {
                    _this5.props.onClick();
                }
                if (handlers.onClick) {
                    handlers.onClick();
                }
            },
            className: targetClassNames,
            id: target.props.id || this.props.id
        }));

        return target;
    };

    Popup.prototype.renderPopupElement = function renderPopupElement() {
        var _this6 = this;

        if (this.isClosing()) {
            return null;
        }

        var element = _react2.default.Children.toArray(this.props.children)[1];
        var style = this.props.popupWidth ? { width: this.props.popupWidth + 'px' } : null;
        return _react2.default.createElement(
            _PopupElement2.default,
            {
                modal: this.isModal(),
                close: this.close,
                id: this.props.id,
                ref: function ref(content) {
                    return _this6._content = content;
                },
                key: 'popup',
                didMount: function didMount(content) {
                    return _this6.props.onContentDidMount(content);
                },
                onUnmount: function onUnmount() {
                    _this6.closeCompletely();
                },
                style: style
            },
            element
        );
    };

    Popup.prototype.renderElement = function renderElement() {
        if (this.isClosed()) {
            return null;
        }

        if (this.props.animate) {
            return _react2.default.createElement(
                _CSSTransitionGroup2.default,
                {
                    ref: 'popup',
                    transitionAppear: true,
                    transitionLeave: true,
                    transitionAppearTimeout: 300,
                    transitionEnterTimeout: 300,
                    transitionLeaveTimeout: 300,
                    transitionName: this.props.animationBaseName
                },
                this.renderPopupElement()
            );
        } else {
            return _react2.default.createElement(
                'div',
                { ref: 'popup' },
                this.renderPopupElement()
            );
        }
    };

    Popup.prototype.getProperties = function getProperties() {
        var attachmentProps = attachmentPositions[this.props.attachment];

        var properties = _extends({}, attachmentProps);

        if (this.isModal()) {
            properties.target = document.body;
            properties.targetModifier = 'visible';
        }

        if (this.props.changeAttachmentDynamically) {
            properties.constraints = [{ to: 'window', attachment: 'together' }];
        } else {
            properties.constraints = [];
        }

        return properties;
    };

    Popup.prototype.render = function render() {
        var _this7 = this;

        var tetherProperties = this.getProperties();

        return _react2.default.createElement(
            _TetherComponent2.default,
            _extends({
                ref: function ref(tether) {
                    return _this7.tether = tether;
                }
            }, tetherProperties, {
                onRepositioned: this.handleRepositioned,
                className: this.props.popupClassName
            }),
            this.renderTarget(),
            this.renderElement()
        );
    };

    return Popup;
}(_react2.default.Component);

Popup.propTypes = PROPERTY_TYPES;
Popup.defaultProps = DEFAULT_PROPS;

Popup.childContextTypes = { addCheck: _propTypes2.default.func, addPopupsToReposition: _propTypes2.default.func };
Popup.contextTypes = { addCheck: _propTypes2.default.func, addPopupsToReposition: _propTypes2.default.func };

Popup.attachmentPositions = attachmentPositions;

exports.default = (0, _OnKeyDownHOC.enhanceWithKeyDown)((0, _OnClickOutsideHOC.enhanceWithClickOutside)(Popup));