'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _reactList = require('react-list');

var _reactList2 = _interopRequireDefault(_reactList);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _index = require('../CheckBox/index');

var _index2 = _interopRequireDefault(_index);

var _utils = require('../util/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    value: _propTypes2.default.arrayOf(_propTypes2.default.number.isRequired),
    options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.number.isRequired,
        displayString: _propTypes2.default.node.isRequired
    })),
    onChange: _propTypes2.default.func,
    focus: _propTypes2.default.shape({
        id: _propTypes2.default.number.isRequired,
        displayString: _propTypes2.default.node.isRequired
    })
};
var DEFAULT_PROPS = {};

var CheckBoxList = function (_React$Component) {
    _inherits(CheckBoxList, _React$Component);

    function CheckBoxList(props) {
        _classCallCheck(this, CheckBoxList);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this.state = {
            selected: (0, _utils.findIdentifiables)(_this.props.options, _this.props.value),
            counter: 0
        };

        _this.renderItem = _this.renderItem.bind(_this);
        _this.renderOptions = _this.renderOptions.bind(_this);
        return _this;
    }

    CheckBoxList.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        this.setState({
            focus: nextProps.focus
        });
        var isValueChanged = nextProps.value !== this.props.value;
        var isOptionsChanged = nextProps.options !== this.props.options;

        if (isValueChanged || isOptionsChanged) {
            this.setState({
                selected: (0, _utils.findIdentifiables)(nextProps.options, nextProps.value),
                counter: this.state.counter + 1
            });
        }
    };

    CheckBoxList.prototype.isSelected = function isSelected(option) {
        var found = (0, _find2.default)(this.state.selected, function (selected) {
            if (option.id === selected.id) {
                return true;
            }
        });

        if (found) {
            return true;
        } else {
            return false;
        }
    };

    CheckBoxList.prototype.select = function select(option) {
        var selected = void 0;

        if (!this.isSelected(option)) {
            selected = [option].concat(this.state.selected);
        } else {
            selected = _immutable2.default.fromJS(this.state.selected).filter(function (item) {
                return item.get('id') !== option.id;
            }).toJS();
        }

        if (!this.isControlled()) {
            this.setState({
                selected: selected,
                counter: this.state.counter + 1
            });
        }

        if (this.props.onChange) {
            this.props.onChange(selected);
        }
    };

    CheckBoxList.prototype.isControlled = function isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    CheckBoxList.prototype.getFocusedEl = function getFocusedEl() {
        return this._focusedEl;
    };

    CheckBoxList.prototype.renderItem = function renderItem(index, key) {
        var _this2 = this;

        var option = this.props.options[index];
        var selected = this.isSelected(option);
        var focus = this.state.focus && this.state.focus.id === option.id;
        var style = (0, _classnames2.default)('check-box-list__item', {
            focused: focus
        });
        return _react2.default.createElement(
            'tr',
            {
                tabIndex: '0',
                ref: function ref(option) {
                    return focus ? _this2._focusedEl = option : null;
                },
                className: style,
                key: key,
                onClick: function onClick() {
                    return _this2.select(option);
                }
            },
            _react2.default.createElement(
                'td',
                { className: 'check-box-list__check-box' },
                _react2.default.createElement(_index2.default, { value: selected })
            ),
            _react2.default.createElement(
                'td',
                { className: 'check-box-list__label', title: option.displayString },
                option.displayString
            )
        );
    };

    CheckBoxList.prototype.renderOptions = function renderOptions(items, ref) {
        return _react2.default.createElement(
            'table',
            { className: 'check-box-list__table' },
            _react2.default.createElement(
                'colgroup',
                null,
                _react2.default.createElement('col', { className: 'check-box-list__check-box-column' }),
                _react2.default.createElement('col', null)
            ),
            _react2.default.createElement(
                'tbody',
                { ref: ref },
                items
            )
        );
    };

    CheckBoxList.prototype.render = function render() {
        return _react2.default.createElement(_reactList2.default, {
            itemRenderer: this.renderItem,
            itemsRenderer: this.renderOptions,
            length: this.props.options.length,
            type: 'uniform',
            justToUpdate: this.state.counter
        });
    };

    return CheckBoxList;
}(_react2.default.Component);

CheckBoxList.propTypes = PROPERTY_TYPES;
CheckBoxList.defaultProps = DEFAULT_PROPS;

exports.default = CheckBoxList;