'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _fuse = require('fuse.js');

var _fuse2 = _interopRequireDefault(_fuse);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _trim = require('lodash/trim');

var _trim2 = _interopRequireDefault(_trim);

var _IconInput = require('../InputIcon/IconInput');

var _IconInput2 = _interopRequireDefault(_IconInput);

var _utils = require('../util/utils');

var _fuseSearchConfig = require('./fuseSearchConfig');

var _fuseSearchConfig2 = _interopRequireDefault(_fuseSearchConfig);

var _WithIdAndTypeHOC = require('../util/hoc/WithIdAndTypeHOC');

var _KeyBoardNav = require('./KeyBoardNav');

var _KeyBoardNav2 = _interopRequireDefault(_KeyBoardNav);

var _SelectInput = require('./SelectInput');

var _PopupWithControl = require('../Popup/PopupWithControl');

var _PopupWithControl2 = _interopRequireDefault(_PopupWithControl);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROPERTY_TYPES = {
    disabled: _propTypes2.default.bool,
    placeholder: _propTypes2.default.string,
    search: _propTypes2.default.bool,
    open: _propTypes2.default.bool,
    onPopupStateChange: _propTypes2.default.func,
    value: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]),
    options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]).isRequired,
        displayString: _propTypes2.default.string.isRequired,
        grayed_item: _propTypes2.default.bool
    })),
    className: _propTypes2.default.string,
    Input: _propTypes2.default.any,
    popup: _propTypes2.default.shape({
        attachment: _propTypes2.default.string,
        className: _propTypes2.default.string,
        animationBaseName: _propTypes2.default.string,
        width: _propTypes2.default.number
    }),
    footer: _propTypes2.default.node
};
var DEFAULT_PROPS = {
    placeholder: 'Select...',
    search: false,
    disabled: false,
    Input: _SelectInput.SingleSelectInput,
    popup: {
        attachment: 'bottom left',
        className: 'select__popover',
        animationBaseName: 'select__popover--animation-slide-y'
    }
};

var Select = function (_React$Component) {
    _inherits(Select, _React$Component);

    function Select(props) {
        _classCallCheck(this, Select);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this._contentDidMount = function (content) {
            if (_this.props.search) {
                setTimeout(function () {
                    _this._searchInput.focus();
                }, 0);
            }
        };

        _this.handlePopupStateChange = function (open) {
            if (_this.props.disabled) {
                return;
            }

            if (!_this.isPopupControlled()) {
                _this.setState({ open: open });
            }

            if (open) {
                _this.handlePopupOpening();
            }

            if (_this.props.onPopupStateChange) {
                _this.props.onPopupStateChange(open);
            }
        };

        _this.renderInput = function (storeRef) {
            var selectedObject = _this.state.selected;
            var text = selectedObject ? selectedObject.displayString : _this.props.placeholder;

            var className = (0, _classnames2.default)(_this.props.className, 'select', {
                placeholder: !selectedObject,
                disabled: _this.props.disabled,
                grayed_item: selectedObject && selectedObject.grayed_item
            });

            var InputComponent = _this.props.Input;

            return _react2.default.createElement(InputComponent, {
                id: _this.props.id,
                onKeyDown: function onKeyDown(e) {
                    return _this.navigate(e);
                },
                inputRef: function inputRef(ref) {
                    storeRef(ref);
                    _this.inputRef = ref;
                },
                value: text,
                className: className
            });
        };

        _this.renderPopup = function () {
            return _react2.default.createElement(
                'div',
                { className: 'select__popup' },
                _this.props.search && _react2.default.createElement(
                    'div',
                    { className: 'select__search-container' },
                    _react2.default.createElement(_IconInput2.default, {
                        inputRef: function inputRef(input) {
                            return _this._searchInput = input;
                        },
                        fluid: true,
                        placeholder: 'Search ... ',
                        size: 'sm',
                        value: _this.state.search,
                        onKeyDown: function onKeyDown(e) {
                            return _this.navigate(e);
                        },
                        onChange: function onChange(value) {
                            return _this.applySearch(value);
                        },
                        className: 'select__search',
                        iconClassName: 'im icon-search'
                    })
                ),
                _react2.default.createElement(
                    'div',
                    { ref: function ref(optionContainer) {
                            return _this._optionContainer = optionContainer;
                        }, className: 'select__options' },
                    _this.renderItems()
                ),
                _this.props.footer ? _react2.default.createElement(
                    'div',
                    { className: 'select__popup-footer' },
                    _this.props.footer
                ) : null
            );
        };

        _this.state = {
            selected: _this.findById(_this.props.options, props.value),
            open: false,
            filtered: props.options,
            search: ''
        };
        _this.fuse = new _fuse2.default(props.options, _fuseSearchConfig2.default);
        return _this;
    }

    Select.prototype.componentWillMount = function componentWillMount() {
        if (this.isPopupControlled()) {
            this.setState({
                open: this.props.open
            });
        }
    };

    Select.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        this.fuse.setCollection(nextProps.options);

        if ((0, _utils.isDefined)(nextProps.value)) {
            this.setState({ selected: this.findById(nextProps.options, nextProps.value) });
        }

        if ((0, _utils.isDefined)(nextProps.open)) {
            this.setState({ open: nextProps.open });
        }

        if (this.props.options !== nextProps.options) {
            this.applySearch(this.state.search, nextProps.options);
        }
    };

    Select.prototype.findById = function findById(options, id) {
        return (0, _find2.default)(options, function (option) {
            return option && option.id === id;
        });
    };

    Select.prototype.select = function select(option) {
        this.inputRef && this.inputRef.focus && this.inputRef.focus();
        if (!this.isControlled()) {
            this.setState({
                selected: option,
                open: false
            });
        } else {
            this.setState({ open: false });
        }

        this.navigate(null, option);

        if (this.props.onChange) {
            this.props.onChange(option.id, option);
        }
    };

    Select.prototype.isControlled = function isControlled() {
        return (0, _utils.isDefined)(this.props.value);
    };

    Select.prototype.handlePopupOpening = function handlePopupOpening() {
        var _this2 = this;

        this.setState({ filtered: this.props.options }, function () {
            if (_this2.refs.selected) {
                var selectedItem = _this2.refs.selected;
                _this2._optionContainer.scrollTop = selectedItem.offsetTop;
            }
        });
    };

    Select.prototype.renderItems = function renderItems() {
        var _this3 = this;

        var selectedObject = this.state.selected;
        var tempSelectionObject = this.state.tempSelection;

        return this.state.filtered.map(function (option) {
            var selected = selectedObject && selectedObject.id === option.id;
            var tempSelection = tempSelectionObject && tempSelectionObject.id === option.id;
            var className = (0, _classnames2.default)('select__option', option.className, {
                selected: tempSelection,
                grayed_item: option.grayed_item
            });

            var ref = selected || tempSelection ? { ref: 'selected' } : null;

            return _react2.default.createElement(
                'div',
                _extends({ tabIndex: '0' }, ref, { className: className, key: option.id, onClick: function onClick() {
                        return _this3.select(option);
                    } }),
                option.displayString
            );
        });
    };

    Select.prototype.applySearch = function applySearch(value) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.props.options;

        var filtered = (0, _trim2.default)(value).length === 0 ? options : this.fuse.search(value);

        this.setState({
            filtered: filtered,
            search: value
        });

        this.resetNav();
    };

    Select.prototype.isPopupControlled = function isPopupControlled() {
        return (0, _utils.isDefined)(this.props.open);
    };

    Select.prototype.getInput = function getInput() {
        if (this.props.search && this._searchInput) {
            return this._searchInput;
        } else {
            return this.inputRef;
        }
    };

    Select.prototype.getSelectedOptionEl = function getSelectedOptionEl() {
        return this.refs.selected;
    };

    Select.prototype.getOptions = function getOptions() {
        return this.state.filtered;
    };

    Select.prototype.render = function render() {
        var _props$popup = this.props.popup,
            attachment = _props$popup.attachment,
            className = _props$popup.className,
            animationBaseName = _props$popup.animationBaseName,
            width = _props$popup.width;


        return _react2.default.createElement(_PopupWithControl2.default, {
            popupOpen: this.state.open,
            handlePopupStateChange: this.handlePopupStateChange,
            popupWidth: width,
            attachment: attachment,
            popupClassName: className,
            animationBaseName: animationBaseName,
            renderInput: this.renderInput,
            renderPopup: this.renderPopup,
            testId: this.props.testId
        });
    };

    return Select;
}(_react2.default.Component);

Select.propTypes = PROPERTY_TYPES;
Select.defaultProps = DEFAULT_PROPS;

exports.default = (0, _WithIdAndTypeHOC.withIdAndTypeContext)((0, _KeyBoardNav2.default)(false)(Select), 'Select');