'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _noop = require('lodash/noop');

var _noop2 = _interopRequireDefault(_noop);

var _lowerCase = require('lodash/lowerCase');

var _lowerCase2 = _interopRequireDefault(_lowerCase);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utils = require('../../util/utils');

var _Popup = require('../../Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _GroupedMultiSelectContent = require('./GroupedMultiSelectContent');

var _GroupedMultiSelectContent2 = _interopRequireDefault(_GroupedMultiSelectContent);

var _GroupSubSection = require('./GroupSubSection');

var _GroupSubSection2 = _interopRequireDefault(_GroupSubSection);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROP_TYPES = {
    onChange: _propTypes2.default.func,
    showSubSection: _propTypes2.default.bool,
    placeholder: _propTypes2.default.string,
    value: _propTypes2.default.object,
    options: _propTypes2.default.object,
    keys: _propTypes2.default.arrayOf(_propTypes2.default.string),
    extra: _propTypes2.default.func,
    subSection: _propTypes2.default.any,
    onGoBack: _propTypes2.default.func,
    onPopupStateChange: _propTypes2.default.func,
    popupHeader: _propTypes2.default.node
};

var DEF_PROPS = {
    autoFocus: true,
    onPopupStateChange: _noop2.default,
    onChange: _noop2.default,
    extra: _noop2.default
};

var GroupedMultiSelect = function (_React$Component) {
    _inherits(GroupedMultiSelect, _React$Component);

    function GroupedMultiSelect(props) {
        _classCallCheck(this, GroupedMultiSelect);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this._isSubSectionControlled = function () {
            return (0, _isUndefined2.default)(_this.props.subSection);
        };

        _this._goBack = function () {
            if (_this._isSubSectionControlled()) {
                _this.props.onGoBack();
            } else {
                _this.setState({
                    showSubSection: false
                });
            }
        };

        _this._defaultTemplate = function (name, count) {
            return name + ' (' + count + ')';
        };

        _this._onPopupStateChange = function (open) {
            _this.setState({
                popupOpen: open,
                tooltipOpen: false
            });

            if (open) {
                if (!_this.props.showSubSection) {
                    _this._sortSelectedOnTop();
                }
                _this._onPopupOpening();
            }

            _this.props.onPopupStateChange(open);
        };

        _this._onTooltipStateChange = function (open) {
            if (!_this.state.popupOpen && !_this._isSelectedEmpty()) {
                _this.setState({ tooltipOpen: open });
            }
        };

        _this._onPopupOpening = function () {
            var popupContent = _reactDom2.default.findDOMNode(_this._popup.getContent());
            var input = _reactDom2.default.findDOMNode(_this._input);
            popupContent.style.width = input.offsetWidth + 'px';
        };

        _this._onChange = function (values) {
            if (_this._isControlled()) {
                _this.props.onChange(values);
            } else {
                _this.setState({
                    selected: values
                });
                _this.props.onChange(values);
            }
        };

        _this._bindChildMethods = function (sortSelectedOnTop) {
            _this._sortSelectedOnTop = sortSelectedOnTop;
        };

        _this._renderTooltip = function (selected) {
            var empty = true;
            var result = [];
            Object.keys(selected).forEach(function (key) {
                var len = selected[key].length;
                if (len !== 0) {
                    empty = false;
                    result.push(_this._defaultTemplate(key, selected[key].length));
                }
            });
            if (empty) {
                return _this.props.placeholder;
            }
            return result.join(',');
        };

        _this.state = {
            showSubSection: false,
            popupOpen: false,
            tooltipOpen: false
        };
        _this._sortSelectedOnTop = null;
        return _this;
    }

    GroupedMultiSelect.prototype._getSelectedValues = function _getSelectedValues() {
        if (this._isControlled()) {
            return this.props.value;
        }
        return this.state.selected;
    };

    GroupedMultiSelect.prototype._getSelectedOption = function _getSelectedOption(id, key) {
        return (0, _find2.default)(this.props.options[key], ['id', id]) || {};
    };

    GroupedMultiSelect.prototype._getSelectedValuesByKey = function _getSelectedValuesByKey(key) {
        return this._getSelectedValues()[key] || [];
    };

    GroupedMultiSelect.prototype._isSelectedEmpty = function _isSelectedEmpty() {
        var _this2 = this;

        return !Object.keys(this._getSelectedValues()).some(function (key) {
            return _this2._getSelectedValuesByKey(key).length > 0;
        });
    };

    GroupedMultiSelect.prototype._isControlled = function _isControlled() {
        return !(0, _isUndefined2.default)(this.props.value);
    };

    GroupedMultiSelect.prototype._showSubSection = function _showSubSection() {
        return this.props.showSubSection || this.state.showSubSection;
    };

    GroupedMultiSelect.prototype._renderDefaultView = function _renderDefaultView() {
        return _react2.default.createElement(_GroupedMultiSelectContent2.default, _extends({}, this.props, {
            bindChildMethods: this._bindChildMethods,
            onChange: this._onChange,
            value: this._getSelectedValues()
        }));
    };

    GroupedMultiSelect.prototype._renderSubSection = function _renderSubSection() {
        if ((0, _isUndefined2.default)(this.props.showSubSection)) {
            throw new Error('Subsection should be provided as property');
        }
        return _react2.default.createElement(
            _GroupSubSection2.default,
            { goBack: this._goBack },
            this.props.subSection
        );
    };

    GroupedMultiSelect.prototype._renderSelectedValues = function _renderSelectedValues() {
        var _this3 = this;

        return _react2.default.createElement(
            'div',
            null,
            this.props.keys.map(function (key) {
                return _this3._getSelectedValuesByKey(key).map(function (id) {
                    return _this3._getSelectedOption(id, key).displayString;
                }).sort(function (l, r) {
                    return (0, _utils.compare)((0, _lowerCase2.default)(l), (0, _lowerCase2.default)(r));
                }).map(function (option, index) {
                    return _react2.default.createElement(
                        'div',
                        { key: key + '-' + index },
                        option
                    );
                });
            })
        );
    };

    GroupedMultiSelect.prototype.render = function render() {
        var _this4 = this;

        var className = (0, _classnames2.default)(this.props.className, 'select__popover');
        return _react2.default.createElement(
            _Popup2.default,
            {
                ref: function ref(popup) {
                    return _this4._popup = popup;
                },
                attachment: 'bottom left',
                on: 'click',
                popupClassName: className,
                open: this.state.popupOpen,
                animationBaseName: 'select__popover--animation-slide-y',
                onPopupStateChange: function onPopupStateChange(open) {
                    return _this4._onPopupStateChange(open);
                }
            },
            _react2.default.createElement(
                _Popup2.default,
                {
                    attachment: 'bottom left',
                    open: this.state.tooltipOpen,
                    on: 'hover',
                    popupClassName: 'tooltip',
                    onPopupStateChange: this._onTooltipStateChange
                },
                _react2.default.createElement(
                    'div',
                    {
                        ref: function ref(input) {
                            return _this4._input = input;
                        },
                        id: this.props.id,
                        tabIndex: '0',
                        className: 'select select--grouped'
                    },
                    this._renderTooltip(this._getSelectedValues()),
                    _react2.default.createElement('i', { className: 'im icon-arrow-down' })
                ),
                this._renderSelectedValues()
            ),
            this._showSubSection() ? this._renderSubSection() : this._renderDefaultView()
        );
    };

    return GroupedMultiSelect;
}(_react2.default.Component);

GroupedMultiSelect.propTypes = PROP_TYPES;
GroupedMultiSelect.defaultProps = DEF_PROPS;

exports.default = GroupedMultiSelect;