'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _lowerCase = require('lodash/lowerCase');

var _lowerCase2 = _interopRequireDefault(_lowerCase);

var _noop = require('lodash/noop');

var _noop2 = _interopRequireDefault(_noop);

var _SelectionList = require('../SelectionList');

var _SelectionList2 = _interopRequireDefault(_SelectionList);

var _IconInput = require('../../InputIcon/IconInput');

var _IconInput2 = _interopRequireDefault(_IconInput);

var _utils = require('../../util/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PROP_TYPES = {
    onChange: _propTypes2.default.func,
    options: _propTypes2.default.object.isRequired,
    groupView: _propTypes2.default.shape({
        noOptionPlaceholder: _propTypes2.default.element
    }),
    value: _propTypes2.default.object,
    extra: _propTypes2.default.func,
    showSearch: _propTypes2.default.bool,
    popupHeader: _propTypes2.default.node
};

var DEF_PROPS = {
    searchPlaceholder: 'Search...',
    groups: [],
    height: 192,
    bindChildMethods: _noop2.default,
    options: [],
    keys: [],
    groupView: {},
    onChange: _noop2.default,
    compare: _utils.compare,
    showSearch: true,
    methodCallback: _noop2.default
};

var GroupedMultiSelectContent = function (_React$Component) {
    _inherits(GroupedMultiSelectContent, _React$Component);

    function GroupedMultiSelectContent(props) {
        _classCallCheck(this, GroupedMultiSelectContent);

        var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

        _this._sortSelectedOnTop = function () {
            var filtered = _this._getFilteredOptions({
                options: _this.props.options,
                sortFn: _this._sortedOnTop
            });

            _this.setState({
                filtered: filtered
            });
        };

        _this._setSelectedOptions = function (selectedIds, key) {
            if (_this._isControlled()) {
                _this.props.onChange(_extends({}, _this.props.value, _defineProperty({}, key, selectedIds)), key);
            } else {
                var selected = _extends({}, _this.state.selected, _defineProperty({}, key, selectedIds));
                _this.setState({
                    selected: selected
                });
                _this.props.onChange(selected, key);
            }
        };

        _this._isOptionSelected = function (_ref, key) {
            var id = _ref.id;

            return _this._getSelected(key).indexOf(id) !== -1;
        };

        _this._sortedOnTop = function (key) {
            return function (option1, option2) {
                var isOption1Selected = _this._isOptionSelected(option1, key);
                var isOption2Selected = _this._isOptionSelected(option2, key);

                if (isOption1Selected && !isOption2Selected) {
                    return -1;
                } else if (isOption2Selected && !isOption1Selected) {
                    return 1;
                } else {
                    return _this.props.compare((0, _lowerCase2.default)(option1.displayString), (0, _lowerCase2.default)(option2.displayString));
                }
            };
        };

        _this._onSelect = function (option, key) {
            var selectedIds = void 0;
            if (_this._isOptionSelected(option, key)) {
                selectedIds = _this._getSelected(key).filter(function (id) {
                    return id !== option.id;
                });
            } else {
                selectedIds = [option.id].concat(_this._getSelected(key));
            }
            _this._setSelectedOptions(selectedIds, key);
        };

        _this._applySearch = function (searchQuery) {
            var filtered = _this._getFilteredOptions({
                options: _this.props.options,
                filterFn: function filterFn(options) {
                    return _this._filterByQueryStr(searchQuery, options);
                },
                sortFn: _this._sortedOnTop
            });

            _this.setState({
                filtered: filtered
            });
        };

        _this._doSearchMatchAnyResult = function (key) {
            if (!_this.props.options[key] || _this.props.options[key].length === 0) {
                return true;
            }
            return _this.state.filtered[key] && _this.state.filtered[key].length !== 0;
        };

        _this._getGroupViewConfig = function (key) {
            return _this.props.groupView[key] || {};
        };

        _this.state = {
            selected: {},
            filtered: {}
        };
        return _this;
    }

    GroupedMultiSelectContent.prototype.componentWillMount = function componentWillMount() {
        this.props.bindChildMethods(this._sortSelectedOnTop);
        this.setState({
            filtered: this._getFilteredOptions({ options: this.props.options })
        });
    };

    GroupedMultiSelectContent.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        var _this2 = this;

        if (nextProps.options && nextProps.options !== this.props.options) {
            var filtered = this._getFilteredOptions({
                keys: nextProps.keys,
                options: nextProps.options,
                filterFn: function filterFn(options) {
                    return _this2._filterByQueryStr(_this2._getSearchValue(), options);
                }
            });

            this.setState({
                filtered: filtered
            });
        }
    };

    GroupedMultiSelectContent.prototype.componentDidMount = function componentDidMount() {
        var x = window.scrollX;
        var y = window.scrollY;
        if (this._searchInput) {
            this._searchInput.focus();
        }
        window.scrollTo(x, y);
    };

    GroupedMultiSelectContent.prototype._getSearchValue = function _getSearchValue() {
        return this._searchInput ? this._searchInput.value : '';
    };

    GroupedMultiSelectContent.prototype._getSelected = function _getSelected(key) {
        if (this._isControlled()) {
            return this.props.value[key] || [];
        }

        return this.state.selected[key] || [];
    };

    GroupedMultiSelectContent.prototype._isControlled = function _isControlled() {
        return !(0, _isUndefined2.default)(this.props.value);
    };

    GroupedMultiSelectContent.prototype._filterByQueryStr = function _filterByQueryStr(queryString, options) {
        return options.filter(function (option) {
            return (0, _utils.contains)(option.displayString, queryString);
        });
    };

    GroupedMultiSelectContent.prototype._getFilteredOptions = function _getFilteredOptions(_ref2) {
        var options = _ref2.options,
            filterFn = _ref2.filterFn,
            sortFn = _ref2.sortFn,
            _ref2$keys = _ref2.keys,
            keys = _ref2$keys === undefined ? this.props.keys : _ref2$keys;

        return keys.reduce(function (result, key) {
            var filtered = options[key];

            if (filterFn) {
                filtered = filterFn(options[key]);
            }

            if (sortFn) {
                filtered = filtered.sort(sortFn(key));
            }
            result[key] = filtered;
            return result;
        }, {});
    };

    GroupedMultiSelectContent.prototype._getHeader = function _getHeader(key) {
        var viewConfig = this._getGroupViewConfig(key);
        var Header = viewConfig.header;

        return Header ? _react2.default.createElement(Header, {
            value: this.props.value[key],
            options: this.props.options[key]
        }) : _react2.default.createElement(
            'span',
            { className: 'select__group-name' },
            key
        );
    };

    GroupedMultiSelectContent.prototype.render = function render() {
        var _this3 = this;

        return _react2.default.createElement(
            'div',
            { className: 'select__popup', style: { fontSize: '1.4rem' } },
            this.props.popupHeader,
            this.props.showSearch ? _react2.default.createElement(
                'div',
                { className: 'select__search-container' },
                _react2.default.createElement(_IconInput2.default, {
                    inputRef: function inputRef(input) {
                        return _this3._searchInput = input;
                    },
                    fluid: true,
                    placeholder: this.props.searchPlaceholder,
                    size: 'sm',
                    onChange: this._applySearch,
                    className: 'select__search',
                    iconClassName: 'im icon-search'
                })
            ) : null,
            _react2.default.createElement(
                'div',
                { style: { maxHeight: this.props.height }, className: 'select__options' },
                this.props.keys.map(function (key) {
                    var viewConfig = _this3._getGroupViewConfig(key);
                    return _react2.default.createElement(
                        'div',
                        { key: key, className: 'select__group' },
                        _react2.default.createElement(
                            'div',
                            { className: 'select__group-header' },
                            _this3._getHeader(key)
                        ),
                        _this3._doSearchMatchAnyResult(key) ? _react2.default.createElement(_SelectionList2.default, {
                            extra: viewConfig.extra,
                            noOptionPlaceholder: viewConfig.noOptionPlaceholder,
                            isSelected: function isSelected(option) {
                                return _this3._isOptionSelected(option, key);
                            },
                            onChange: function onChange(option) {
                                return _this3._onSelect(option, key);
                            },
                            value: _this3._getSelected(key),
                            options: _this3.state.filtered[key]
                        }) : _react2.default.createElement(
                            'div',
                            { className: 'no-results-found' },
                            'No results found'
                        )
                    );
                })
            )
        );
    };

    return GroupedMultiSelectContent;
}(_react2.default.Component);

GroupedMultiSelectContent.propType = PROP_TYPES;
GroupedMultiSelectContent.defaultProps = DEF_PROPS;

exports.default = GroupedMultiSelectContent;