'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _some = require('lodash/some');

var _some2 = _interopRequireDefault(_some);

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _Row = require('./Row/Row');

var _Row2 = _interopRequireDefault(_Row);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DESC = 'DESC';
var ASC = 'ASC';

var PROPERTY_TYPES = {
    className: _propTypes2.default.string,
    data: _propTypes2.default.array,
    sorted: _propTypes2.default.shape({
        key: _propTypes2.default.any,
        direction: _propTypes2.default.any
    }),
    colgroup: _propTypes2.default.array,
    colSpanBottom: _propTypes2.default.number,
    defSorting: _propTypes2.default.number,
    headerCells: _propTypes2.default.func,
    loadingIndicator: _propTypes2.default.func,
    rowStyle: _propTypes2.default.func,
    rowProps: _propTypes2.default.func,
    row: _propTypes2.default.object,
    cells: _propTypes2.default.func.isRequired,
    rowIndex: _propTypes2.default.func.isRequired,
    bottomSection: _propTypes2.default.element,
    onHeaderClick: _propTypes2.default.func,
    testId: _propTypes2.default.any
};
var DEFAULT_PROPS = {
    data: [],
    headerCells: function headerCells() {
        return [];
    }
};

var Table = function (_React$Component) {
    _inherits(Table, _React$Component);

    function Table() {
        var _temp, _this, _ret;

        _classCallCheck(this, Table);

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this._toggleDirection = function () {
            return _this._getSortDirection() === ASC ? DESC : ASC;
        }, _temp), _possibleConstructorReturn(_this, _ret);
    }

    Table.prototype._getCells = function _getCells(obj) {
        return this.props.cells(obj);
    };

    Table.prototype._handleRowClick = function _handleRowClick(item, index) {
        if (this.props.row.onClick) {
            this.props.row.onClick(item, this.props.rowIndex(item), index);
        }
    };

    Table.prototype._showLoadingIndicator = function _showLoadingIndicator(show) {
        if (this.props.loadingIndicator && show) {
            return this.props.loadingIndicator();
        }
        return show ? _react2.default.createElement(
            'span',
            null,
            'Loading...'
        ) : null;
    };

    Table.prototype._createBottomSection = function _createBottomSection(create, item) {
        var rowDetails = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

        var newProps = _extends({}, item);

        if (!(0, _isEmpty2.default)(rowDetails)) {
            newProps = _extends({}, newProps, rowDetails);
        }

        return create ? _react2.default.cloneElement(this.props.bottomSection, _extends({}, newProps)) : null;
    };

    Table.prototype._renderBottomSection = function _renderBottomSection(showLoader, hasData, item, rowDetails) {
        return _react2.default.createElement(
            'tr',
            { className: 'ros-table__bottom-section' },
            _react2.default.createElement(
                'td',
                { colSpan: this.props.colSpanBottom || 0 },
                this._showLoadingIndicator(showLoader && !hasData),
                this._createBottomSection(hasData, item, rowDetails)
            )
        );
    };

    Table.prototype._sortIndicator = function _sortIndicator(key) {
        return key === this._getSortIndex() ? _react2.default.createElement(
            'span',
            { className: 'sort-indicator' },
            this._getSortDirection() === DESC ? _react2.default.createElement(
                'span',
                null,
                '\u2193'
            ) : _react2.default.createElement(
                'span',
                null,
                '\u2191'
            )
        ) : null;
    };

    Table.prototype._getSortIndex = function _getSortIndex() {
        return this.props.sorted && this.props.sorted.key;
    };

    Table.prototype._isSortControlled = function _isSortControlled() {
        return !(0, _isUndefined2.default)(this.props.sorted);
    };

    Table.prototype._getSortDirection = function _getSortDirection() {
        var dir = this.props.sorted && this.props.sorted.direction;
        return dir && dir.toUpperCase();
    };

    Table.prototype._handleHeaderCellClick = function _handleHeaderCellClick(key, index, cell, sortable) {
        if (this.props.onHeaderClick && sortable) {
            this.props.onHeaderClick(key, index, cell, this._toggleDirection());
        }
    };

    Table.prototype._renderHeader = function _renderHeader() {
        var _this2 = this;

        return this.props.headerCells().map(function (headerCell, index) {
            var cellStyle = (0, _classnames2.default)('ros-table__header-cell');

            if (headerCell === null) {
                return _react2.default.createElement('th', { className: cellStyle, key: index });
            }

            var sortable = headerCell.sortable === undefined ? true : headerCell.sortable;

            var isSorted = headerCell.key === _this2._getSortIndex();
            var style = (0, _classnames2.default)({
                'ros-table__header-sortable': _this2._isSortControlled() && sortable,
                sorted: isSorted
            });

            return _react2.default.createElement(
                'th',
                _extends({ key: index, className: cellStyle }, headerCell.props || {}),
                _react2.default.createElement(
                    'span',
                    {
                        onClick: function onClick(e) {
                            return _this2._handleHeaderCellClick(headerCell.key, index, headerCell, sortable);
                        },
                        className: style
                    },
                    headerCell.el
                ),
                isSorted && _this2._sortIndicator(headerCell.key)
            );
        });
    };

    Table.prototype._getRowStyle = function _getRowStyle(item) {
        if (this.props.rowStyle) {
            return this.props.rowStyle(item);
        }

        return {};
    };

    Table.prototype._getRowProps = function _getRowProps(item) {
        if (this.props.rowProps) {
            return this.props.rowProps(item);
        }

        return {};
    };

    Table.prototype._findRowDetails = function _findRowDetails(item) {
        var _this3 = this;

        return (0, _find2.default)(this.props.rowDetails, function (obj) {
            return obj.id === _this3.props.rowIndex(item);
        });
    };

    Table.prototype._renderRow = function _renderRow(item, index) {
        var _this4 = this;

        var loadingDetails = (0, _some2.default)(this.props.rowDetails, { id: this.props.rowIndex(item) });

        var rowDetails = this._findRowDetails(item);
        var hasData = !(0, _isEmpty2.default)(rowDetails && rowDetails.data);

        var showLoader = loadingDetails && !hasData;

        var cellStyle = (0, _classnames2.default)('ros-table__cell');

        var style = (0, _classnames2.default)('ros-table__row', {
            'row--expanded': loadingDetails || hasData
        }, this._getRowStyle(item));

        return _react2.default.createElement(
            _react2.default.Fragment,
            null,
            _react2.default.createElement(
                _Row2.default,
                {
                    className: style,
                    key: index,
                    item: item,
                    onClick: function onClick(e) {
                        return _this4._handleRowClick(item, index);
                    },
                    rowProps: this._getRowProps(item)
                },
                this._getCells(item).map(function (cell, key) {
                    if (cell === null) {
                        return _react2.default.createElement('td', { className: cellStyle, key: key });
                    }

                    if (_react2.default.isValidElement(cell)) {
                        return _react2.default.createElement(
                            'td',
                            { key: key, className: cellStyle },
                            cell
                        );
                    }

                    return _react2.default.createElement(
                        'td',
                        _extends({ key: key, className: cellStyle }, cell.props || {}),
                        cell.el
                    );
                })
            ),
            this._renderBottomSection(showLoader, hasData, item, rowDetails)
        );
    };

    Table.prototype._renderColGroup = function _renderColGroup() {
        if (this.props.colgroup) {
            return _react2.default.createElement(
                'colgroup',
                null,
                this.props.colgroup.map(function (col, index) {
                    return _react2.default.createElement('col', { key: index, width: col });
                })
            );
        }
        return null;
    };

    Table.prototype._getDefComparator = function _getDefComparator() {
        if (!(0, _isUndefined2.default)(this.props.defSorting)) {
            var hCells = this.props.headerCells();
            if (!(0, _isArray2.default)(hCells)) {
                throw new Error('Expected the headerCells returns array');
            } else {
                var comparator = hCells[this.props.defSorting].comparator;
                if (!(0, _isFunction2.default)(comparator)) {
                    throw new Error('Expected comparator to be a function.Or you forgot to provide comparator fn');
                }

                return comparator;
            }
        }

        return null;
    };

    Table.prototype.render = function render() {
        var _this5 = this;

        var style = (0, _classnames2.default)('ros-table', this.props.className);

        return _react2.default.createElement(
            'table',
            { 'data-test-id': this.props.testId, className: style },
            this._renderColGroup(),
            _react2.default.createElement(
                'thead',
                null,
                _react2.default.createElement(
                    'tr',
                    { className: 'ros-table__header' },
                    this._renderHeader()
                )
            ),
            _react2.default.createElement(
                'tbody',
                null,
                this.props.data.map(function (item, index) {
                    return _this5._renderRow(item, index);
                })
            )
        );
    };

    return Table;
}(_react2.default.Component);

Table.order = {
    ASC: ASC,
    DESC: DESC
};
Table.propTypes = PROPERTY_TYPES;
Table.defaultProps = DEFAULT_PROPS;

exports.default = Table;