'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.parseDDMMYYYY = exports.formatMonthYearHeader = exports.formatShortMonthYear = exports.formatShortMonth = exports.formatD = exports.formatWeekDay = exports.formatDDMMYYYY = exports.formatRange = exports.formatFullMonthYear = exports.formatFullMonth = exports.formatDMonthTime = exports.formatDMonthYear = exports.DD_MM_YYYY = undefined;

var _dateUtils = require('./date-utils');

//01.12.2016 =>
var D = 'D'; // 1
var D_MONTH = 'D MMM'; // 1 Dec
var D_MONTH_YEAR = 'D MMM YYYY'; // 1 Dec 2016
var D_MONTH_TIME = 'D MMM HH:mm'; // 1 Dec 16:25
var SHORT_MONTH = 'MMM'; // Dec
var FULL_MONTH = 'MMMM'; // December
var MMM_YYYY = 'MMM YYYY'; // Dec 2016
var MONTH_YEAR_HEADER = 'MMMM, YYYY'; // December, 2016
var FULL_MONTH_YEAR = 'MMMM YYYY'; //December 2016
var DD_MM_YYYY = 'DD.MM.YYYY'; // 01.12.2016
var WEEK_DAY = 'dd'; // Th

function formatDMonthYear(date) {
    var hideYear = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

    if ((0, _dateUtils.isToday)(date)) {
        return 'Today';
    }
    if (hideYear && (0, _dateUtils.isCurrentYear)(date)) {
        return (0, _dateUtils.format)(date, D_MONTH);
    } else {
        return (0, _dateUtils.format)(date, D_MONTH_YEAR);
    }
}

function formatDMonthTime(date) {
    return (0, _dateUtils.format)(date, D_MONTH_TIME);
}

function formatFullMonth(date) {
    return (0, _dateUtils.format)(date, FULL_MONTH);
}

function formatDDMMYYYY(date) {
    return (0, _dateUtils.format)(date, DD_MM_YYYY);
}

function formatWeekDay(date) {
    return (0, _dateUtils.format)(date, WEEK_DAY);
}

function formatD(date) {
    return (0, _dateUtils.format)(date, D);
}

function formatFullMonthYear(date) {
    if ((0, _dateUtils.isCurrentYear)(date)) {
        return formatFullMonth(date);
    } else {
        return (0, _dateUtils.format)(date, FULL_MONTH_YEAR);
    }
}

function formatShortMonth(month) {
    return (0, _dateUtils.format)(month, SHORT_MONTH);
}

function formatShortMonthYear(month) {
    return (0, _dateUtils.format)(month, MMM_YYYY);
}

function formatMonthYearHeader(month) {
    return (0, _dateUtils.format)(month, MONTH_YEAR_HEADER);
}

function parseDDMMYYYY(date) {
    return (0, _dateUtils.parse)(date, DD_MM_YYYY);
}

function formatRange(from, to) {
    var hideYear = (0, _dateUtils.isCurrentYear)(from) && (0, _dateUtils.isYearTheSame)(from, to);
    var fromFormatted = (0, _dateUtils.isMonthTheSame)(from, to) ? (0, _dateUtils.format)(from, D) : formatDMonthYear(from, hideYear);
    var toFormatted = formatDMonthYear(to, hideYear);
    return fromFormatted + ' - ' + toFormatted;
}

exports.DD_MM_YYYY = DD_MM_YYYY;
exports.formatDMonthYear = formatDMonthYear;
exports.formatDMonthTime = formatDMonthTime;
exports.formatFullMonth = formatFullMonth;
exports.formatFullMonthYear = formatFullMonthYear;
exports.formatRange = formatRange;
exports.formatDDMMYYYY = formatDDMMYYYY;
exports.formatWeekDay = formatWeekDay;
exports.formatD = formatD;
exports.formatShortMonth = formatShortMonth;
exports.formatShortMonthYear = formatShortMonthYear;
exports.formatMonthYearHeader = formatMonthYearHeader;
exports.parseDDMMYYYY = parseDDMMYYYY;