'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getConfiguredMoment = exports.clone = exports.format = exports.parse = exports.addYears = exports.addMonths = exports.addWeeks = exports.addDays = exports.setYear = exports.setMonth = exports.getFirstDaysOfAllMonths = exports.getMonths = exports.getFirstDayOfPreviousWeek = exports.getFirstDayOfWeek = exports.getLastDayOfYear = exports.getFirstDayOfYear = exports.getLastDayOfMonth = exports.getFirstDayOfMonth = exports.getWeekDayByIndex = exports.getYear = exports.getMonth = exports.getToday = exports.isDayAfterOrTheSame = exports.isDayAfter = exports.isDayBeforeOrTheSame = exports.isDayBefore = exports.isYearTheSame = exports.isMonthAfterOrTheSame = exports.isMonthAfter = exports.isMonthBeforeOrTheSame = exports.isMonthBefore = exports.isMonthTheSame = exports.isDayTheSame = exports.isBetweenNotInclusive = exports.isAfter = exports.isCurrentYear = exports.isCurrentMonth = exports.isToday = exports.isFirstDayOfWeek = exports.isWeekend = exports.isDateBetween = exports.isDateBefore = exports.isDatesSame = exports.isValidDate = exports.DD_MM_YYYY = undefined;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _range = require('lodash/range');

var _range2 = _interopRequireDefault(_range);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_moment2.default.updateLocale('en', {
    week: {
        dow: 1,
        doy: 6
    }
});

var DD_MM_YYYY = exports.DD_MM_YYYY = 'DD.MM.YYYY';
function isWeekend(date) {
    var dayOfWeek = date.isoWeekday();
    return dayOfWeek === 6 || dayOfWeek === 7;
}

function isFirstDayOfWeek(date) {
    return date.day() === _moment2.default.localeData().firstDayOfWeek();
}

function isToday(date) {
    return isDayTheSame((0, _moment2.default)(), date);
}

function isCurrentMonth(date) {
    return isMonthTheSame((0, _moment2.default)(), date);
}

function isCurrentYear(date) {
    return isYearTheSame((0, _moment2.default)(), date);
}

function isAfter(dateToCheck, dateToCheckAgainst) {
    return dateToCheck.isAfter(dateToCheckAgainst);
}

function isBetweenNotInclusive(dateToCheck, from, to) {
    return dateToCheck.isBetween(from, to);
}

function isDayTheSame(date1, date2) {
    if (!date2) {
        return false;
    }

    return date1.isSame(date2, 'day');
}

function isMonthTheSame(date1, date2) {
    return date1.isSame(date2, 'month');
}

function isMonthBefore(date1, date2) {
    return date1.isBefore(date2, 'month');
}

function isMonthBeforeOrTheSame(date1, date2) {
    return date1.isSameOrBefore(date2, 'month');
}

function isMonthAfter(date1, date2) {
    return date1.isAfter(date2, 'month');
}

function isMonthAfterOrTheSame(date1, date2) {
    return date1.isSameOrAfter(date2, 'month');
}

function isYearTheSame(date1, date2) {
    return date1.isSame(date2, 'year');
}

function getToday() {
    return (0, _moment2.default)();
}

function getWeekDayByIndex(index) {
    return (0, _moment2.default)().isoWeekday(index);
}

function getFirstDayOfMonth(date) {
    return date.clone().startOf('month');
}

function getLastDayOfMonth(date) {
    return date.clone().endOf('month');
}

function getFirstDayOfYear(date) {
    return date.clone().startOf('year');
}

function getLastDayOfYear(date) {
    return date.clone().endOf('year');
}

function getFirstDayOfPreviousWeek(date) {
    return getFirstDayOfWeek(addWeeks(date, -1));
}

function getFirstDayOfWeek(date) {
    return date.clone().startOf('week');
}

function getMonths() {
    return _moment2.default.months();
}

function getFirstDaysOfAllMonths(year) {
    var firstMonth = getFirstDayOfYear(setYear((0, _moment2.default)(), year));
    return (0, _range2.default)(12).map(function (value) {
        return addMonths(firstMonth, value);
    });
}

function addDays(date, amount) {
    return date.clone().add(amount, 'days');
}

function addWeeks(date, amount) {
    return date.clone().add(amount, 'weeks');
}

function addMonths(date, amount) {
    return date.clone().add(amount, 'months');
}

function addYears(date, amount) {
    return date.clone().add(amount, 'years');
}

function parse(date, format) {
    return (0, _moment2.default)(date, format, true); //strict
}

function format(date, format) {
    return date.format(format);
}

function clone(date) {
    return date.clone();
}

function getConfiguredMoment() {
    return _moment2.default;
}

function setYear(date, year) {
    return date.clone().set('year', year);
}

function getYear(date) {
    return date.clone().get('year');
}

function setMonth(date, month) {
    return date.clone().set('month', month);
}

function getMonth(date) {
    return date.clone().get('month');
}

function isDayBefore(date1, date2) {
    return date1.isBefore(date2, 'day');
}

function isDayBeforeOrTheSame(date1, date2) {
    return date1.isSameOrBefore(date2, 'day');
}

function isDayAfter(date1, date2) {
    return date1.isAfter(date2, 'day');
}

function isDateBetween(date1, date2, date) {
    var pattern = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : '[]';

    return date.isBetween(date1, date2, null, pattern);
}

function isDateBefore(dateAgainst, date) {
    return dateAgainst.isBefore(date);
}
function isDayAfterOrTheSame(date1, date2) {
    return date1.isSameOrAfter(date2, 'day');
}

function isDatesSame(date1, date2) {
    return date1.isSame(date2);
}

function isValidDate(str, format) {
    var d = parse(str, format);
    return d !== null && d.isValid();
}

exports.isValidDate = isValidDate;
exports.isDatesSame = isDatesSame;
exports.isDateBefore = isDateBefore;
exports.isDateBetween = isDateBetween;
exports.isWeekend = isWeekend;
exports.isFirstDayOfWeek = isFirstDayOfWeek;
exports.isToday = isToday;
exports.isCurrentMonth = isCurrentMonth;
exports.isCurrentYear = isCurrentYear;
exports.isAfter = isAfter;
exports.isBetweenNotInclusive = isBetweenNotInclusive;
exports.isDayTheSame = isDayTheSame;
exports.isMonthTheSame = isMonthTheSame;
exports.isMonthBefore = isMonthBefore;
exports.isMonthBeforeOrTheSame = isMonthBeforeOrTheSame;
exports.isMonthAfter = isMonthAfter;
exports.isMonthAfterOrTheSame = isMonthAfterOrTheSame;
exports.isYearTheSame = isYearTheSame;
exports.isDayBefore = isDayBefore;
exports.isDayBeforeOrTheSame = isDayBeforeOrTheSame;
exports.isDayAfter = isDayAfter;
exports.isDayAfterOrTheSame = isDayAfterOrTheSame;
exports.getToday = getToday;
exports.getMonth = getMonth;
exports.getYear = getYear;
exports.getWeekDayByIndex = getWeekDayByIndex;
exports.getFirstDayOfMonth = getFirstDayOfMonth;
exports.getLastDayOfMonth = getLastDayOfMonth;
exports.getFirstDayOfYear = getFirstDayOfYear;
exports.getLastDayOfYear = getLastDayOfYear;
exports.getFirstDayOfWeek = getFirstDayOfWeek;
exports.getFirstDayOfPreviousWeek = getFirstDayOfPreviousWeek;
exports.getMonths = getMonths;
exports.getFirstDaysOfAllMonths = getFirstDaysOfAllMonths;
exports.setMonth = setMonth;
exports.setYear = setYear;
exports.addDays = addDays;
exports.addWeeks = addWeeks;
exports.addMonths = addMonths;
exports.addYears = addYears;
exports.parse = parse;
exports.format = format;
exports.clone = clone;
exports.getConfiguredMoment = getConfiguredMoment;